<?php
require_once 'lib/vod/vod.php';
require_once 'lib/abstract_preloaded_regular_screen.php';

class SmartSearchScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'search_screen';

    public static function get_media_url_str($class_name, $arr=null){
		$arr['screen_id'] = self::ID;
		$arr['class_name'] = $class_name;
        return MediaURL::encode($arr);
    }

    ///////////////////////////////////////////////////////////////////////

    private $vod;
	private $searchMediaUrls;
	
    public function __construct(Vod $vod){
        $this->vod = $vod;
        parent::__construct(self::ID,
           $this->get_folder_views());

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    
    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies){
		// $this->view_idx = isset($plugin_cookies->{"screen.".self::ID.".view_idx"}) ? $plugin_cookies->{"screen.".self::ID.".view_idx"} : 0;
		
		$actions[GUI_EVENT_KEY_SETUP] = ActionFactory::open_folder('setup','Настройки');
		
		$add_params['action'] = 'search';
		$add_params['search_actions'] = 'open';
        $actions[GUI_EVENT_KEY_ENTER] = UserInputHandlerRegistry::create_action($this, 'classAction', $add_params);
		$add_params['action'] = 'search';
		$add_params['search_actions'] = 'keyboard';
        $actions[GUI_EVENT_KEY_PLAY] = UserInputHandlerRegistry::create_action($this, 'classAction', $add_params);
		
		$add_action = UserInputHandlerRegistry::create_action($this, 'searchSetup');
		$add_action['caption'] = 'Настройки поиска';
		$actions[GUI_EVENT_KEY_A_RED]	= $add_action;
		
		$add_action = UserInputHandlerRegistry::create_action($this, 'delSearch');
        $add_action['caption'] = 'Удалить';
		$actions[GUI_EVENT_KEY_D_BLUE] = $add_action;
		
        $add_action = UserInputHandlerRegistry::create_action($this, 'up_item');
		$add_action['caption'] = 'Поднять';
		$actions[GUI_EVENT_KEY_B_GREEN] = $add_action;
		
		$add_action = UserInputHandlerRegistry::create_action($this, 'down_item');
		$add_action['caption'] = 'Опустить';
		$actions[GUI_EVENT_KEY_C_YELLOW] = $add_action;
		
        $actions[GUI_EVENT_KEY_POPUP_MENU] = UserInputHandlerRegistry::create_action($this, 'popup_menu');
		$useNumKey = HD::getItemsArr('useNumKey');
		foreach ($useNumKey as $k => $v){
			$params['openCat'] = $v;
			$actions[SmartConfig::$keyArr[$k]] = UserInputHandlerRegistry::create_action($this, 'add_actions', $params);
		}
        return $actions;
    }

    public function get_handler_id(){ 
		return self::ID; 
	}

    private function get_update_action(&$user_input, &$plugin_cookies){
        $parent_media_url = MediaURL::decode($user_input->parent_media_url);
        $range = HD::create_regular_folder_range( $this->get_all_folder_items( $parent_media_url, $plugin_cookies));
        return ActionFactory::update_regular_folder( $range, true, 1);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies){
		if ($user_input->control_id == 'classAction'){
			$defs = array();
			hd_silence_warnings();
			if (isset($user_input->indiClass)){
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				$class_name = $user_input->indiClass;//$selected_media_url->class_name;
				return $class_name::classAction ($defs, $user_input, $this->select_url);
			}
			return SmartConfig::classAction ($defs, $user_input, $this->select_url);
		}
		if ($user_input->control_id == 'up_item'){
            if (!isset($user_input->selected_media_url))
                return null;
			$selected_media_url = MediaURL::decode($user_input->selected_media_url);
			if (!isset($selected_media_url->searchStr))
				return null;
			$searchHistori = HD::getItemsArr('searchHistori');
			$k = array_search($user_input->selected_media_url, $searchHistori);
			if ($k !== false && $k !== 0){
				$t = $searchHistori[$k - 1];
				$searchHistori[$k - 1] = $searchHistori[$k];
				$searchHistori[$k] = $t;
				HD::saveItemsArr('searchHistori', $searchHistori);
				$parent_media_url = MediaURL::decode($user_input->parent_media_url);
				$range = HD::create_regular_folder_range(
					$this->get_all_folder_items(
						$parent_media_url, $plugin_cookies));
				return ActionFactory::update_regular_folder(
					$range, false, $user_input->sel_ndx - 1);
			}
			return null;
        }
		if ($user_input->control_id == 'down_item'){
            if (!isset($user_input->selected_media_url))
                return null;
			$selected_media_url = MediaURL::decode($user_input->selected_media_url);
			if (!isset($selected_media_url->searchStr))
				return null;
			$searchHistori = HD::getItemsArr('searchHistori');
			$k = array_search($user_input->selected_media_url, $searchHistori);
			if ($k !== false && $k !== count($searchHistori) - 1){
				$t = $searchHistori[$k + 1];
				$searchHistori[$k + 1] = $searchHistori[$k];
				$searchHistori[$k] = $t;
				HD::saveItemsArr('searchHistori', $searchHistori);
				$parent_media_url = MediaURL::decode($user_input->parent_media_url);
				$range = HD::create_regular_folder_range(
					$this->get_all_folder_items(
						$parent_media_url, $plugin_cookies));
				return ActionFactory::update_regular_folder(
					$range, false, $user_input->sel_ndx + 1);
			}
			return null;
		}
		if ($user_input->control_id == 'delSearch'){
            if (!isset($user_input->selected_media_url))
                return null;
			$selected_media_url = MediaURL::decode($user_input->selected_media_url);
			if (!isset($selected_media_url->searchStr))
				return null;
			$searchHistori = HD::getItemsArr('searchHistori');
			$searchHistori_flip = array_flip($searchHistori);
			$mUrlHist = $this->searchMediaUrls[$user_input->selected_media_url];
			if (isset($searchHistori_flip[$mUrlHist])){
				unset($searchHistori[$searchHistori_flip[$mUrlHist]]);
				$searchHistori = array_values($searchHistori);
				HD::saveItemsArr('searchHistori', $searchHistori);
			}
            return ActionFactory::invalidate_folders(array($user_input->parent_media_url));
		}		
		if ($user_input->control_id == 'searchSetup'){
			$searchSettings = (HD::getItemsArr('searchSettings') != array()) ? HD::getItemsArr('searchSettings') : array('useOsk' => 1, 'searchСhoice' => 1, 'kpSearchMetod' => 0);
			$add_params['control_id']= $user_input->control_id;
			if ((isset($user_input->selected_control_id))&&(isset($user_input->action_type))){
				$searchSettings['useOsk'] = $user_input->useOsk;
				$searchSettings['searchСhoice'] = $user_input->searchСhoice;
				$searchSettings['kpSearchMetod'] = $user_input->kpSearchMetod;
				if (HD::saveItemsArr('searchSettings', $searchSettings) == false)
					return ActionFactory::show_title_dialog("Сохранение не удалось");
				$params['initial_sel_ndx'] = $user_input->initial_sel_ndx;
				$action = UserInputHandlerRegistry::create_action($this, $user_input->control_id, $params);
				return ActionFactory::close_dialog_and_run($action);
			}
			$osk_ops[0] = 'Нет';
			$osk_ops[1] = 'Стандартная';
			$osk_ops[2] = 'Продвинутая (текстовая)';
			ControlFactory::add_label($defs, '',  'Экранная клавиатура в поиске:');
			$add_params['initial_sel_ndx'] = 0;
			ControlFactory::add_combobox($defs, $this, $add_params,
				'useOsk',
				'',
				$searchSettings['useOsk'], $osk_ops,  650, true
			);
			$show_ops[1] = 'Да';
			$show_ops[0] = 'Нет';
			ControlFactory::add_label($defs, '',  'Выбирать сайт поиска:');
			$add_params['initial_sel_ndx'] = $add_params['initial_sel_ndx'] + 1;
			ControlFactory::add_combobox($defs, $this, $add_params,
				'searchСhoice',
				'',
				$searchSettings['searchСhoice'], $show_ops,  650, true
			);
			$kpSearchMethod_ops[0] = 'Вариант 1';
			$kpSearchMethod_ops[1] = 'Вариант 2';
			$kpSearchMethod_ops[2] = 'Вариант 3';
			ControlFactory::add_label($defs, '',  'Глобальный поиск:');
			$add_params['initial_sel_ndx'] = $add_params['initial_sel_ndx'] + 1;
			ControlFactory::add_combobox($defs, $this, $add_params,
				'kpSearchMetod',
				'',
				$searchSettings['kpSearchMetod'], $kpSearchMethod_ops,  650, true
			);
			ControlFactory::add_vgap($defs, -10);
			ControlFactory::add_smart_label($defs, "", '<text color="23" size="tiny">*Вариант 1 быстрый, но иногда не работает</text>');
			ControlFactory::add_vgap($defs, -40);
			ControlFactory::add_smart_label($defs, "", '<text color="23" size="tiny">**Вариант 2 медленный, может не работать изза капчи</text>');
			ControlFactory::add_vgap($defs, -40);
			ControlFactory::add_smart_label($defs, "", '<text color="23" size="tiny">***Вариант 3 быстрый, но малое количество итемов в выдаче</text>');
			ControlFactory::add_vgap($defs, -40);
			ControlFactory::add_smart_label($defs, "", '<text color="23" size="tiny">Рекомендуется использовать вариант 1 - остальные резерв</text>');
			$attrs['initial_sel_ndx'] = isset($user_input->initial_sel_ndx) ? $user_input->initial_sel_ndx : -1;
			return ActionFactory::show_dialog('Настройки поиска', $defs, true, 750, $attrs);
		}		
		if ($user_input->control_id == 'popup_menu'){
            if (!isset($user_input->selected_media_url))
                return null;
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Удалить',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'delSearch'),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_0')
			);
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Поднять',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'up_item'),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_queue_move_up')
			);
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Опустить',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'down_item'),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_queue_move_down')
			);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Настройки поиска',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'searchSetup'),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_setup')
			);
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$add_params['action'] = 'background_actions';
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Задать бекграунд',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_add_background')
			);
			if (file_exists(HD::get_data_path() . "/icons/" . self::ID . "_bg.jpg")){
				$add_params['action'] = 'background_actions';
				$add_params['del'] = true;
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'Удалить бекграунд',
					GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_del_background')
				);
			}
			return ActionFactory::show_popup_menu($menu_items);
        }
        return null;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies){
        $this->vod->folder_entered($media_url, $plugin_cookies);
		if (isset($media_url->categoryId)){
			$class_name = $media_url->class_name;
			$pfx = 'Поиск на ' . str_replace(" Трейлеры", "", $class_name::$cName);
		}else{
			$class_name = 'Kinopoisk_Class';
			$pfx = 'Глобальный поиск по базе Кинопоиска';
		}
        $items = array();
		if (file_exists('/data/data/com.example.dunevoice')){
			$item[PluginRegularFolderItem::caption] = '[Искать фильм/сериал]';
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::icon_path] = SmartConfig::getPluginIco('videoSearchVoice');
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_icon_path] = SmartConfig::getPluginIco($class_name);
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_info] = '||Голосовой поиск фильма или сериала|' . $pfx;
			$item[PluginRegularFolderItem::media_url] = SmartVodListScreen::get_media_url_str($class_name, array('searchType' => 'film', 'voice' => true));
			array_push($items, $item);
			if (!isset($media_url->categoryId)){
				$item[PluginRegularFolderItem::caption] = '[Искать актера/режиссера/сценариста]';
				$item[PluginRegularFolderItem::view_item_params][ViewItemParams::icon_path] = SmartConfig::getPluginIco('peopleSearchVoice');
				$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_info] = '||Голосовой поиск актера или режиссера или сценариста|Глобальный поиск по базе Кинопоиска';
				$item[PluginRegularFolderItem::media_url] = SmartVodListScreen::get_media_url_str('Kinopoisk_Class', array('searchType' => 'people', 'voice' => true));
				array_push($items, $item);
			}
		}
		$item[PluginRegularFolderItem::caption] = '[Искать фильм/сериал]';
		$item[PluginRegularFolderItem::view_item_params][ViewItemParams::icon_path] = SmartConfig::getPluginIco('videoSearch');
		$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_icon_path] = SmartConfig::getPluginIco($class_name);
		$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_info] = '||Искать фильм или сериал|' . $pfx;
		$item[PluginRegularFolderItem::media_url] = SmartVodListScreen::get_media_url_str($class_name, array('searchType' => 'film'));
		
        array_push($items, $item);
		if (!isset($media_url->categoryId)){
			$item[PluginRegularFolderItem::caption] = '[Искать актера/режиссера/сценариста]';
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::icon_path] = SmartConfig::getPluginIco('peopleSearch');
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_info] = '||Искать актера или режиссера или сценариста|Глобальный поиск по базе Кинопоиска';
			$item[PluginRegularFolderItem::media_url] = SmartVodListScreen::get_media_url_str('Kinopoisk_Class', array('searchType' => 'people'));
			array_push($items, $item);
		}
		$searchHistori = HD::getItemsArr('searchHistori');
		$sUrl = MediaURL::decode($class_name::getSearchMediaUrlStr($class_name, array()));
		$nClassName = $class_name;
        foreach ($searchHistori as $mUrlHist){
			$media_url = MediaURL::decode($mUrlHist);
			$class_name = $media_url->class_name;
			$raw = json_decode($mUrlHist, true);
			$raw['screen_id'] = $sUrl->screen_id;
			$raw['class_name'] = $nClassName;
			$mUrl = MediaURL::encode ($raw);
			$this->searchMediaUrls[$mUrl] = $mUrlHist;
			if (isset($media_url->searchType))
				$icon_path = SmartConfig::getPluginIco('searchHistori' . $media_url->searchType);
			else
				$icon_path = SmartConfig::getPluginIco('searchHistorivideo');
			$detailed_icon = SmartConfig::getPluginIco($class_name);
			if ($media_url->searchStr == false){
				hd_print ('serchStrUrl:' . $mUrlHist);
				continue;
			}
			if (class_exists($class_name)){
				$n1 = str_replace(" Трейлеры", "", $class_name::$cName);
				$item_caption_color = 23;
			}else{
				$detailed_icon = SmartConfig::getPluginIco(0);
				$n1 = '||Каталог (источник) ' . str_replace("_Class", "", $class_name) . ' плагином больше не поддерживается';
				$item_caption_color = 17;
			}
			$info = '||Поиск|' . $media_url->searchStr . '||' . $n1 . '|Кнопка плей редактирование поиска';
			$item[PluginRegularFolderItem::caption] = $media_url->searchStr;
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::icon_path] = $icon_path;
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_icon_path] = $detailed_icon;
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_detailed_info] = $info;
			$item[PluginRegularFolderItem::view_item_params][ViewItemParams::item_caption_color] = $item_caption_color;
			$item[PluginRegularFolderItem::media_url] = $mUrl;
			array_push($items, $item);
        }
        return $items;
    }
	private function get_folder_views(){
		$background_path = SmartConfig::getPluginIco(self::ID . '_bg');
		$paintBackground = (HD::getItemsArr('paintBackground') != array()) ? HD::getItemsArr('paintBackground') : array('paint_widget_background' => true, 'paint_content_box_background' => true, 'paint_path_box_background' => true, 'paint_details_box_background' => true);
		$item_detailed_info_font_size = (HD::getItemStr('detailedInfoFontSize') != '') ? HD::getItemStr('detailedInfoFontSize') : FONT_SIZE_SMALL;
		
		$arr[0][PluginRegularFolderView::async_icon_loading] = false;
		
		$arr[0][PluginRegularFolderView::view_params][ViewParams::num_cols] = 1;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::num_rows] = 12;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_details] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_path]= $background_path;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_order] = 0;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_height] = 1080;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_width] = 1920;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::optimize_full_screen_background] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_item_info_in_details] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_auto_line_break] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_title_color] = 10;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_text_color] = 15;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_font_size] = $item_detailed_info_font_size;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_widget_background] = $paintBackground['paint_widget_background'];
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_content_box_background] = $paintBackground['paint_content_box_background'];
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_path_box_background] = $paintBackground['paint_path_box_background'];
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_details_box_background] = $paintBackground['paint_details_box_background'];
		
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_width] = 50;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_height] = 50;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_path] = 'missing://';
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_layout] = HALIGN_LEFT;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_valign] = VALIGN_CENTER;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_dx] = 10;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_dy] = -5;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_caption_font_size] = FONT_SIZE_NORMAL;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_caption_width] = 1100;

		$arr[0][PluginRegularFolderView::not_loaded_view_item_params] = array();

		return $arr;
    }
    public function get_archive(MediaURL $media_url){
        return $this->vod->get_archive($media_url);
    }
	public function meinMenuOpenFolderDialog(&$user_input, &$plugin_cookies){
		hd_restore_warnings();
		if (Kinopoisk_Class::checkKpAccess() == 'false'){
			$alertOff = (HD::getItemStr(Kinopoisk_Class::$cName . 'checkKpAccessAlertOff') != '') ? HD::getItemStr(Kinopoisk_Class::$cName . 'checkKpAccessAlertOff') : false;
			$add_params['action'] = $user_input->action;
			if ((!isset($user_input->closed_dialog))&&($alertOff == false)){
				$add_params['closed_dialog']= true;
				ControlFactory::add_multiline_label($defs, '', "Для полноценного использования плагином необходим доступ к ".Kinopoisk_Class::$cName."!",5);		
				ControlFactory::add_close_dialog_and_apply_button($defs, $this, array ('action' => 'proxy', 'addProxy' => 'socks.zaborona.help:1488', 'addProxyType' => 'SOCKS5'), 'classAction', 'Прокси для ' . Kinopoisk_Class::$cName, 700);
				ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, $user_input->control_id, 'Продолжить без Кинопоиска', 700);
				$add_params['alertOff']= true;
				ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, $user_input->control_id, 'Больше не показывать', 700);
				return  ActionFactory::show_dialog('Внимание!',$defs, true,710,$attrs);
			}
			if (isset($user_input->alertOff))
				if (HD::saveItemStr(self::$cName . 'checkKpAccessAlertOff', 1) == false)
					return ActionFactory::show_title_dialog("Сохранение не удалось");
		}
		if (HD::is_android() == true){
			if (!file_exists('/data/data/com.example.dunevoice')){
				if (isset($user_input->dontAsk))
					$plugin_cookies->duneVoiceAsk = 1;
				if (isset($user_input->install)){
					$post_action = ActionFactory::open_folder();
					return ActionFactory::download_and_install_apk('DuneVoice', 'com.example.dunevoice', array('http://dune-club.info/plugins/update/_other/DuneVoice.apk'), false, $post_action);
				}
				$duneVoiceAsk = isset($plugin_cookies->duneVoiceAsk) ? $plugin_cookies->duneVoiceAsk : 0;
				if ($duneVoiceAsk === 0){
					$title = 'Установить андроид приложение DuneVoice?';
					$add_params['install'] = true;
					$yes_action = UserInputHandlerRegistry::create_action($this, $user_input->control_id, $add_params);
					
					$add_params1['dontAsk'] = true;
					$yes_action1 = UserInputHandlerRegistry::create_action($this, $user_input->control_id, $add_params1);
					$msgs1[] = "Позднее DuneVoice можно уcтановить через настройки плагина или из apk файла";
					$no_action = ActionFactory::show_yes_no_dialog('Больше не спрашивать?', $yes_action1, true, $msgs1, false, 950);
					
					$msgs[] = "DuneVoice необходимо для голосового поиска";
					$msgs[] = "Для работы DuneVoice необходимо необходимо установить гугл сервисы (устанавливаются совместно с плеймаркетом)";
					return ActionFactory::show_yes_no_dialog($title, $yes_action, true, $msgs, $no_action, 950);
				}
			}
		}
		return ActionFactory::open_folder();
	}
}

