<?php
///////////////////////////////////////////////////////////////////////////
abstract class VodListScreen extends AbstractRegularScreen
    implements UserInputHandler {
    ///////////////////////////////////////////////////////////////////////
	static public $total;
    private $vod;

    public function __construct(Vod $vod, $screenID, $folder_views){
        parent::__construct($screenID, $folder_views);
        $this->vod = $vod;
		$this->screenID = $screenID;
        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }
	
    public function get_action_map(MediaURL $media_url, &$plugin_cookies){
		$this->view_idx = isset($plugin_cookies->{"screen.".$this->screenID.".view_idx"}) ? $plugin_cookies->{"screen.".$this->screenID.".view_idx"} : 0;
		$params['event_key'] = 'enter';
		$actions[GUI_EVENT_KEY_ENTER]		= UserInputHandlerRegistry::create_action($this, 'add_actions',$params);
		$params['event_key'] = 'play';
		$actions[GUI_EVENT_KEY_PLAY] 		= UserInputHandlerRegistry::create_action($this, 'add_actions',$params);

		$actions[GUI_EVENT_KEY_POPUP_MENU] = UserInputHandlerRegistry::create_action($this, 'popup_menu');

		$add_action = UserInputHandlerRegistry::create_action($this, 'classAction', array('action' => 'search'));
        $add_action['caption'] = 'Поиск';
		$actions[GUI_EVENT_KEY_SEARCH] = $add_action;
        $actions[GUI_EVENT_KEY_B_GREEN] = $add_action;

		$add_params['action'] = 'addToMainMenu';
		$add_action = UserInputHandlerRegistry::create_action($this, 'classAction', $add_params);
        $add_action['caption'] = 'В Главное меню';
        $actions[GUI_EVENT_KEY_C_YELLOW] = $add_action;

		$add_params['action'] = 'addToFav';
		$add_action = UserInputHandlerRegistry::create_action($this, 'classAction', $add_params);
        $add_action['caption'] = 'Избранное';
        $actions[GUI_EVENT_KEY_D_BLUE] = $add_action;
		$actions[GUI_EVENT_KEY_INFO] = UserInputHandlerRegistry::create_action($this, 'info');
		$actions[GUI_EVENT_KEY_SETUP] = ActionFactory::open_folder('setup','Настройки');

		$class_name = $media_url->class_name;
		$actionsArr = $class_name::getScreenActionMapArr($media_url);
		foreach ($actionsArr as $k => $v){
			$params = false;
			if (isset($v['params']))
				$params = $v['params'];
			$add_action = UserInputHandlerRegistry::create_action($this, $v['create_action'], $params);
			if (isset($v['caption']))
				$add_action['caption'] = $v['caption'];
			$actions[$v['key']] = $add_action;
		}
		$useNumKey = HD::getItemsArr('useNumKey');
		foreach ($useNumKey as $k => $v){
			$params['openCat'] = $v;
			$actions[SmartConfig::$keyArr[$k]] = UserInputHandlerRegistry::create_action($this, 'add_actions', $params);
		}
		return $actions;
    }

    public function get_handler_id(){
		return $this->screenID;
	}
	public function add_actions($user_input) {
		$parent_media_url = MediaURL::decode($user_input->parent_media_url);
		$selected_media_url = MediaURL::decode($user_input->selected_media_url);
		if (isset($parent_media_url->searchKpId)){
			$this->vod->clear_cached_movie();
			return UserInputHandlerRegistry::create_action($this, 'classAction', array ('action' => 'searchKpId', 'addId' => $selected_media_url->movie_id));
		}
		if (isset($parent_media_url->setMyShowId)){
			$this->vod->clear_cached_movie();
			return UserInputHandlerRegistry::create_action($this, 'classAction', array ('action' => 'setMyShowId', 'addId' => $selected_media_url->movie_id));
		}
		if (isset($user_input->openCat)){
			$user_input->selected_media_url = $user_input->openCat; 
			$selected_media_url = MediaURL::decode($user_input->selected_media_url);
			return ActionFactory::open_folder($user_input->selected_media_url, $selected_media_url->caption);
		}
		
		if (isset($this->vod->currentFav)){
			if ($parent_media_url->class_name != 'myShows_Class')
			unset($this->vod->currentFav);
		}
		if ($selected_media_url->screen_id =="vod_series")
			return ActionFactory::vod_play();
		
		return ActionFactory::open_folder();
	}
    public function handle_user_input(&$user_input, &$plugin_cookies){
		if ($user_input->control_id == 'info'){
			if (!isset($user_input->selected_media_url))
                return null;
            $selected_media_url = MediaURL::decode($user_input->selected_media_url);
			if (!isset($selected_media_url->detailed_info))
                return null;
			$strs = explode("|", $selected_media_url->detailed_info);
			foreach ($strs as $k => $str){
				if ($str != ''){
					ControlFactory::add_multiline_label($defs, '', $str, 5);
					ControlFactory::add_vgap($defs, -10);
				}
			}
			ControlFactory::add_close_dialog_button($defs, 'OK', 250);
			return  ActionFactory::show_dialog('INFO', $defs, true, 850, $attrs);
		}
		if ($user_input->control_id == 'add_actions')
			return $this->add_actions($user_input);
		if ($user_input->control_id == 'ifx'){
			$this->vod->ifx = true;
			return ActionFactory::invalidate_folders(array($user_input->inv), $post_action);
		}
		if ($user_input->control_id == 'classAction'){
			$defs = array();
			hd_silence_warnings();
			if (isset($user_input->indiClass)){
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				$class_name = $user_input->indiClass;//$selected_media_url->class_name;
				return $class_name::classAction ($defs, $user_input, $this->select_url);
			}
			return SmartConfig::classAction ($defs, $user_input, $this->select_url);
		}
		if ($user_input->control_id == 'classSetupTorrServer'){
			$defs = array();
			hd_silence_warnings();
			return TorrServer::setup($defs, $user_input);
		}
		if ($user_input->control_id == 'classSetupPeerflixServer'){
			$defs = array();
			hd_silence_warnings();
			return PeerflixServer::setup($defs, $user_input);
		}
		if ($user_input->control_id == 'classSetupTransmission'){
			$defs = array();
			hd_silence_warnings();
			return Transmission::setup($defs, $user_input);
		}
		if ($user_input->control_id == 'classSetupTransmissionUPnP'){
			$defs = array();
			hd_silence_warnings();
			return TransmissionUPnP::setup($defs, $user_input);
		}
		if ($user_input->control_id == 'playDirectly'){
			if (!isset($user_input->selected_media_url))
                return null;
			$start_index = $post_action = null;
			$selected_media_url = MediaURL::decode($user_input->selected_media_url);
			$video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;
			$url = DuneSystem::$properties['plugin_www_url'] . 'bazinga.m3u';
			$sPath = '/tmp/www/plugins/' . DuneSystem::$properties['plugin_name'] . '/bazinga.m3u';
			$m3u = "#EXTM3U";
			$m3u .= "\n#EXTINF:-1," . $selected_media_url->name;
			$m3u .= "\n" .  $selected_media_url->link;
			file_put_contents($sPath, $m3u);
			$viewedItemsArr = HD::getItemsArr('viewedItemsArr');
			if (isset($this->vod->currentFav)){
				$currentFav = MediaURL::decode($this->vod->currentFav);
			}
			if (isset($selected_media_url->series_id)){
				$viewedItemsArr[$selected_media_url->series_id] = 'viewed';
				HD::saveItemsArr('viewedItemsArr', $viewedItemsArr);
				if (isset($user_input->is_android)){
					$selIdx = $user_input->sel_ndx;
					if (self::$downViewed == true)
						$selIdx++;
					$parent_media_url = MediaURL::decode($user_input->parent_media_url);
					$range = $this->get_folder_range($parent_media_url, 0, $plugin_cookies);
					$post_action = ActionFactory::update_regular_folder($range, true, $selIdx);
					if (isset($sfa2))
						$post_action = ActionFactory::invalidate_folders(array($this->vod->currentFav), $post_action);
				}else{
					$post_action = ActionFactory::launch_media_url($url, $post_action, array('start_index' => $start_index));
					$invArr[] = $user_input->parent_media_url;
					if (isset($sfa2))
						$invArr[] = $this->vod->currentFav;
					return ActionFactory::invalidate_folders($invArr, $post_action);
				}
			}
			file_put_contents($sPath, $m3u);
			if (isset($user_input->is_android)){
				if (!isset($user_input->playAll))
					$url = $selected_media_url->link;
				return HD::get_android_action($url, 'video', $post_action);
			}
			return ActionFactory::launch_media_url($url, null, array('start_index' => $start_index, 'caption' => $selected_media_url->caption));
		}
		if ($user_input->control_id == 'popup_menu'){
			if (isset($user_input->selected_media_url)){
				$add_params['action'] = 'addToFav';
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'Избранное',
					GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_addToFav')
				);
				$menu_items [] =  array(
					GuiMenuItemDef::is_separator => true,);
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'Поиск',
					GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', array('action' => 'search')),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_ssearch')
				);
				$menu_items [] =  array(
					GuiMenuItemDef::is_separator => true,);
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				$class_name = $selected_media_url->class_name;
				if (preg_match("|_Class|",$class_name)){
					$mItems = $class_name::getPopUpMenuItems($selected_media_url, $user_input->parent_media_url);
					foreach ($mItems as $v){
						if ($v == 'is_separator')
							$menu_items [] =  array(
							GuiMenuItemDef::is_separator => true,);
						else if (isset($v['conID']))
							$menu_items[] = array(
								GuiMenuItemDef::caption => $v['caption'],
								GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, $v['conID'], $v['params']),
								GuiMenuItemDef::icon_url => $v['icon_url'],
							);
						else
							$menu_items[] = array(
								GuiMenuItemDef::caption => $v['caption'],
								GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $v['params']),
								GuiMenuItemDef::icon_url => $v['icon_url'],
							);
					}
				}
			}
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$add_params['action'] = 'background_actions';
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Задать бекграунд',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_add_background')
			);
			if (file_exists(HD::get_data_path() . "/icons/" . $this->screenID . "_bg.jpg")){
				$add_params['action'] = 'background_actions';
				$add_params['del'] = true;
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'Удалить бекграунд',
					GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_del_background')
				);
			}
			return ActionFactory::show_popup_menu($menu_items);
		}
        return null;
    }
    ///////////////////////////////////////////////////////////////////////

    protected function get_short_movie_range(MediaURL $media_url, $from_ndx, &$plugin_cookies){
		$class_name = $media_url->class_name;
		if(isset($media_url->searchStr))
			$movies = $class_name::getSearchShortMovieRange($media_url, $this->vod->m_feed);
		else
			$movies = $class_name::getShortMovieRange ($media_url, $this->vod->m_feed);
		if(!count($movies))
			return new ShortMovieRange(0, 0);
        self::$total += count($movies);
        return new ShortMovieRange($from_ndx, self::$total, $movies);
    }
	public function get_folder_range(MediaURL $media_url, $from_ndx, &$plugin_cookies){
        $movie_range = $this->get_short_movie_range($media_url, $from_ndx, $plugin_cookies);
        $total = intval($movie_range->total);
        if ($total <= 0)
            return HD::create_regular_folder_range(array());
		if ($movie_range->from_ndx > 0 && $total == 0){
			return  HD::create_regular_folder_range(
				$items,
				$movie_range->from_ndx > 0 ? $movie_range->from_ndx - 1 : 0,
				$movie_range->from_ndx ,
				false
			);
        }
        $items = array();
		
		$left = SmartConfig::getPluginIco('left');
        foreach ($movie_range->short_movies as $short_movie){
			$color = 15;
			$class_name = $media_url->class_name;
			$mArr = $view_item_params = array();
			$mArr['caption'] = $short_movie->name;
			if (isset($short_movie->shape))
				$mArr['shape'] = $short_movie->shape;
			if (isset($media_url->hCaptions))
				$mArr['hCaptions'] = $media_url->hCaptions;
			if (isset($media_url->caption))
				$mArr['hCaptions'][] = $media_url->caption;
			if (isset($short_movie->argument))
				$mArr['argument'] = $short_movie->argument;
			$mArr['poster_url'] = $short_movie->poster_url;
			if (isset($short_movie->goToScreen)){
				$goToScreen = $short_movie->goToScreen;
				if ($goToScreen == 'VodSeriesListScreen'){
					$mArr['movie_id'] = $short_movie->id;
					if (isset($short_movie->caption_original))
						$mArr['caption_original'] = $short_movie->caption_original;
					if (isset($short_movie->year))
						$mArr['year'] = $short_movie->year;
					if (isset($short_movie->description))
						$mArr['description'] = $short_movie->description;
				}else
					$mArr['categoryId'] = $short_movie->id;
				if (isset($short_movie->favMyShows))
					$mArr['favMyShows'] = $short_movie->favMyShows;
				if (isset($short_movie->mShowsA))
					$mArr['mShowsA'] = $short_movie->mShowsA;
				if (isset($short_movie->link))
					$mArr['link'] = $short_movie->link;
				$mUrl = $goToScreen::get_media_url_str($class_name, $mArr);
			}else if (isset($short_movie->tType)){
				$mArr['movie_id'] = $media_url->movie_id;
				$mArr['torrent_id'] = $short_movie->id;
				$mArr['tType'] = $short_movie->tType;
				$mArr['detailed_info'] = $short_movie->info;
				if (isset($short_movie->links))
					$mArr['links'] = $short_movie->links;
				if (isset($short_movie->goToSelfScreen)){
					$goToScreen = $short_movie->goToSelfScreen;
					$mArr['goToSelfScreen'] = true;
					$mUrl = $goToScreen::get_media_url_str($class_name, $mArr);
				}else
					$mUrl = SmartVodTorrentsScreen::get_media_url_str($class_name, $mArr);
			}else{
				$mArr['movie_id'] = $short_movie->id;
				if (isset($short_movie->media_type))
					$mArr['media_type'] = $short_movie->media_type;
				if (isset($short_movie->newClassName))
					$class_name = $short_movie->newClassName;
				if (isset($short_movie->caption_original))
					$mArr['caption_original'] = $short_movie->caption_original;
				if (isset($short_movie->year))
					$mArr['year'] = $short_movie->year;
				if (isset($short_movie->description))
					$mArr['description'] = $short_movie->description;
				if (isset($short_movie->links))
					$mArr['links'] = $short_movie->links;
				if ($short_movie->id == 'notrt')
					$mUrl = 'notrt';
				else if (isset($short_movie->goToSourceList))
					$mUrl = VodSourceListScreen::get_media_url_str($class_name, $mArr);
				else
					$mUrl = VodMovieScreen::get_media_url_str($class_name, $mArr);
			}
			if ((isset($short_movie->left_top))||(isset($short_movie->right_top))||(isset($short_movie->left_bottom))||(isset($short_movie->right_bottom))||
			(isset($short_movie->left_top_img))||(isset($short_movie->right_top_img))||(isset($short_movie->left_bottom_img))||(isset($short_movie->right_bottom_img))){
				$geoArr = $imgArr = $txtArr = array();
				if ((defined('ViewItemParams::item_sticker') === true)&&($this->view_idx < 2)){
					if (isset($short_movie->left_top)){
						$txtArr[0] = $short_movie->left_top;
						$geoArr['txt_color'][0] = 23;
						$geoArr['txt_x'][0] = 15; 
						$geoArr['txt_y'][0] = 10;
						$geoArr['txt_halign'][0] = 'left';
					}
					if (isset($short_movie->right_top)){
						$txtArr[1] = $short_movie->right_top;
						$geoArr['txt_color'][1] = 23;
						$geoArr['txt_x'][1] = -10; 
						$geoArr['txt_y'][1] = 10;
						$geoArr['txt_halign'][1] = 'right';
					}
					if (isset($short_movie->left_bottom)){
						$txtArr[2] = $short_movie->left_bottom;
						$geoArr['txt_color'][2] = 23;
						$geoArr['txt_x'][2] = 15; 
						$geoArr['txt_y'][2] = 10;
						$geoArr['txt_halign'][2] = 'left';
						$geoArr['txt_valign'][2] = 'bottom';
					}
					if (isset($short_movie->right_bottom)){
						$txtArr[3] = $short_movie->right_bottom;
						$geoArr['txt_color'][3] = 23;
						$geoArr['txt_x'][3] = 15; 
						$geoArr['txt_y'][3] = 10;
						$geoArr['txt_halign'][3] = 'right';
						$geoArr['txt_valign'][3] = 'bottom';
					}
					if (isset($short_movie->left_top_img)){
						$imgArr[0] = $short_movie->left_top_img;
						$geoArr['img_x'][0] = 3; 
						$geoArr['img_y'][0] = 7;
						$geoArr['img_halign'][0] = 'left';
					}
					if (isset($short_movie->right_top_img)){
						$imgArr[1] = $short_movie->right_top_img;
						$geoArr['img_x'][1] = -2; 
						$geoArr['img_y'][1] = 2;
						$geoArr['img_halign'][1] = 'right';
					}
					if (isset($short_movie->left_bottom_img)){
						$imgArr[2] = $short_movie->left_bottom_img;
						$geoArr['img_x'][2] = 2; 
						$geoArr['img_y'][2] = 47;
						$geoArr['img_halign'][2] = 'left';
						$geoArr['img_valign'][2] = 'bottom';
					}
					if (isset($short_movie->right_bottom_img)){
						$imgArr[3] = $short_movie->right_bottom_img;
						$geoArr['img_x'][3] = 7; 
						$geoArr['img_y'][3] = 7;
						$geoArr['img_halign'][3] = 'right';
						$geoArr['img_valign'][3] = 'bottom';
					}
					$view_item_params[ViewItemParams::item_sticker] = json_encode(HD::get_sticker($imgArr, $txtArr, $geoArr));
				}
			}
			if((defined('ViewItemParams::item_extra_info'))&&(($this->view_idx > 1)||(($this->screenID == 'vod_simplelist')||($this->screenID == 'vod_torrentlist')))){
				//if((defined('ViewItemParams::item_extra_info'))&&(($this->view_idx > 1)||((($this->screenID == 'vod_simplelist')||($this->screenID == 'vod_torrentlist'))&&($this->view_idx == 0)))){
				if (isset($short_movie->extra_item)){
					$extra_item = array('type' => 'text', 'color' => '23', 'text' => $short_movie->extra_item);
					$item_extra_info = array(
						'items' => array(
						array('type' => 'gap', 'width' => 10),
						$extra_item,
					));
					$view_item_params[ViewItemParams::item_extra_info] = json_encode($item_extra_info);
					$view_item_params[ViewItemParams::item_caption_dy]=-10;
					$view_item_params[ViewItemParams::item_caption_sel_dy]=-10;
				}
			}
			$poster_url = $short_movie->poster_url;
			if (isset($short_movie->icon_path))
				$poster_url = $short_movie->icon_path;
			if ((defined('ViewParams::details_box_width') === true)&&
				(!preg_match('|plugin_file|', $short_movie->poster_url))&&
				(isset($short_movie->details_box_width))
				){
				$view_item_params[ViewItemParams::item_detailed_icon_width] = 200;
				$view_item_params[ViewItemParams::item_detailed_icon_height] = 290;
			}
			if (isset($short_movie->color_str))
				$color = $short_movie->color_str;
			$view_item_params[ViewItemParams::item_caption_color] = $color;
			$view_item_params[ViewItemParams::icon_path] = $poster_url;
			$view_item_params[ViewItemParams::item_detailed_icon_path] = $short_movie->poster_url;
			$view_item_params[ViewItemParams::item_detailed_info] = $short_movie->info;
			if ((defined('ViewItemParams::item_detailed_info_sticker') === true)&&(isset($short_movie->info_sticker)))
				$view_item_params[ViewItemParams::item_detailed_info_sticker] = $short_movie->info_sticker;
            $items[] = array(
                PluginRegularFolderItem::media_url => $mUrl,
                PluginRegularFolderItem::caption => $short_movie->name,
                PluginRegularFolderItem::view_item_params => $view_item_params
            );
            $this->vod->set_cached_short_movie(new ShortMovie($short_movie->id, $short_movie->name, $short_movie->poster_url));
        }
        return HD::create_regular_folder_range($items, $movie_range->from_ndx, $total, true);
    }

    public function get_archive(MediaURL $media_url){
        return $this->vod->get_archive($media_url);
    }
	
	public function get_folder_view(MediaURL $media_url, &$plugin_cookies){
		$this->clear_video_cache();
        $this->vod->m_feed = 0;
		return parent::get_folder_view($media_url, $plugin_cookies);
	}
	
	private function clear_video_cache() {
        self::$total = 0;
        $this->vod->clear_movie_cache();
    }
    
}

///////////////////////////////////////////////////////////////////////////
?>
