<?php

require_once 'lib/abstract_gcomps_screen.php';

abstract class BaseEntityScreen extends AbstractGCompsScreen
{
    protected static $DARKEN_COLOR = "#C0000000";

    protected static $DEFAULT_TRANSITION_STEPS = 5;
    protected static $DEFAULT_SCROLL_STEPS = 8;

    ///////////////////////////////////////////////////////////////////////

    protected static $HGAP;
    protected static $RIGHT_PANEL_WIDTH;
    protected static $RIGHT_PANEL_X;
    protected static $RIGHT_PANEL_HEIGHT;

    protected static $TOP_PANEL_HEIGHT;
    protected static $CENTER_HEADER_HEIGHT;

    protected static $RIGHT_BG_X;
    protected static $RIGHT_BG_WIDTH;

    protected static $LEFT_BG_WIDTH;

    private static function static_init()
    {
        self::$HGAP = L::$PANEL_H_GAP / 2;
        self::$RIGHT_PANEL_X =
            L::$SCREEN_LEFT + L::$VOD_POSTER_WIDTH + L::$PANEL_H_GAP;
        self::$RIGHT_PANEL_WIDTH =
            L::$WIDTH - L::$SCREEN_RIGHT - self::$RIGHT_PANEL_X;
        self::$RIGHT_PANEL_HEIGHT = L::$HEIGHT -
            L::$VOD_PANEL_TOP - L::$PANEL_BOTTOM_OFFSET;

        self::$TOP_PANEL_HEIGHT = L::$VOD_BUTTON_HEIGHT * 4 + L::$PAR_VGAP_BIG +
            L::$VOD_DESCRIPTION_HR_TOP + L::$VOD_DESCRIPTION_HR_BOTTOM;
        self::$CENTER_HEADER_HEIGHT = 12 + L::$VOD_BUTTON_HEIGHT * 4;

        self::$RIGHT_BG_X = self::$RIGHT_PANEL_X - self::$HGAP; 
        self::$RIGHT_BG_WIDTH = L::$WIDTH - self::$RIGHT_BG_X;
        
        self::$LEFT_BG_WIDTH =
            L::$SCREEN_LEFT + L::$VOD_POSTER_WIDTH + self::$HGAP;
    }

    ///////////////////////////////////////////////////////////////////////

    protected $L = null;

    protected $stack;

    public function __construct($id, $stack)
    {
        parent::__construct($id);

        $this->stack = $stack;

        if (!isset(self::$RIGHT_PANEL_X))
            self::static_init();
    }

    ///////////////////////////////////////////////////////////////////////

    public function initialize()
    {
        $this->L = $this->get_layout_data();
    }

    ///////////////////////////////////////////////////////////////////////

    protected abstract function is_first_button_id($button_id);
    protected abstract function is_desc_button_id($button_id);
    protected abstract function get_right_panel_width();

    ///////////////////////////////////////////////////////////////////////

    protected abstract function with_darkened_bg();

    protected abstract function get_button_caption_by_id($id);

    protected abstract function do_get_center_geom($sel_state);

    protected abstract function do_get_poster_info($sel_state);

    protected abstract function do_get_right_panel_children($sel_state);

    protected abstract function do_get_item_geom($sel_state,
        $item_pos, $align_bottom_num_seps);
    protected abstract function do_get_item_sep_geom($sel_state,
        $sep_pos, $align_bottom_num_seps);
    protected abstract function do_get_item_img_geom($selected, $sel_state);

    protected abstract function do_get_item_text_color2($selected, $item_ndx);

    protected abstract function get_window_bg_info();

    protected abstract function handle_enter(&$media_url, &$plugin_cookies);
    protected abstract function handle_other_action(
        &$user_input, &$media_url, &$plugin_cookies);

    ///////////////////////////////////////////////////////////////////////

    protected function is_long_enter()
    { return false; }

    protected function get_layout_data()
    { return (object) array(); }

    protected function is_slideshow_enabled(&$user_input)
    { return false; }

    protected function do_get_top_pane_def($sel_state)
    { return null; }
    protected function do_get_top_pane_geom($sel_state)
    { return null; }

    protected function do_get_center_bg_geom($sel_state)
    { return null; }

    protected function do_get_object_info_panel_children($sel_state)
    { return null; }

    protected function is_partially_darkened($sel_state)
    { return false; }

    protected function do_get_hints_pane_children($sel_state)
    {
        $defs = array();
        $geom = GCompGeom::place_top_left(-1, L::$VOD_BUTTON_HEIGHT);

        $this->do_get_desc_hints_pane_children($defs, $geom, $sel_state);

        return $defs;
    }

    ///////////////////////////////////////////////////////////////////////

    protected function get_num_seps_in_range($from, $to)
    {
        $n = 0;
        foreach ($this->cur_object_list_seps as $ndx => $msg)
            if ($ndx >= $from && $ndx < $to)
                $n++;
        return $n;
    }

    protected function get_align_bottom_num_seps(
        $item_top, $item_sel, $num_items, $page_size)
    {
        if ($item_top + $page_size != $num_items)
            return 0;

        $n = $this->get_num_seps_in_range(
            $item_top, $item_top + $page_size);
        $bwd_pos = $num_items - 1 - $item_sel;
        return max(0, $n - $bwd_pos);
    }

    ///////////////////////////////////////////////////////////////////////

    protected function do_add_description_defs(&$defs, $description,
        $width, $y, $max_chars=999999)
    {
        if ($description == '')
            $description = T::t("msg_no_description");

        $pos = 0;
        $num_chars = 0;
        while ($pos !== false)
        {
            $next_pos = strpos($description, "\n", $pos);

            $par = $next_pos === false ? substr($description, $pos) :
                substr($description, $pos, $next_pos - $pos);
            $par = trim($par);

            if ($par !== '')
            {
                if ($num_chars >= $max_chars)
                    $par = '(...)';

                $geom = $pos == 0 ?
                    GCompGeom::place_top_left($width, -1, 0, $y) :
                    GCompGeom::place_below_left($width, -1, 0, 0);
                $defs[] = GCompsFactory::get_ttf_label_def($geom,
                    null, $par, 0, "#FFAFAFA0", L::$TEXT_SIZE);

                if ($num_chars >= $max_chars)
                    break;

                $num_chars += mb_strlen($par);
            }

            $pos = $next_pos === false ? false : $next_pos + 1;
        }
    }

    protected function do_get_description_panel($description, $top_offset)
    {
        $defs = array();
        $h = self::$RIGHT_PANEL_HEIGHT - $top_offset - L::$PAR_VGAP_SMALL;
        $geom = GCompGeom::place_top_left(
            L::$VOD_PEOPLE_PANEL_WIDTH, $h,
            0, $top_offset + L::$PAR_VGAP_SMALL);
        $this->do_add_description_defs($defs, $description,
            L::$VOD_PEOPLE_PANEL_WIDTH, 0, 800);
        return GCompsFactory::get_vertical_spane_def("center_spane",
            $geom, null, $defs);
    }

    ///////////////////////////////////////////////////////////////////////

    protected function base_get_favorite_panel_children($lines, $sel_state)
    {
        $y = -5;
        $defs = array();

        foreach ($lines as $line)
        {
            $defs[] = GCompsFactory::get_ttf_label_def(
                GCompGeom::place_center(
                    -1,
                    L::$LIST_TEXT_SIZE + L::$PAR_VGAP_SMALL * 2,
                    0, $y - L::$PAR_VGAP_SMALL),
                null, $line, 1, "#FFAFAFA0", L::$LIST_TEXT_SIZE);
            $y += L::$VOD_BUTTON_HEIGHT + L::$PAR_VGAP_SMALL;
        }

        return array(
            GCompsFactory::get_panel_def("center_pane",
                $this->do_get_center_geom($sel_state),
                null, $defs));
    }

    ///////////////////////////////////////////////////////////////////////

    protected function do_get_item_text_size($selected, $sel_state)
    {
        return $selected ? L::$LIST_TEXT_SIZE : L::$TEXT_SIZE;
    }

    protected static function do_get_item_text_color($selected)
    {
        if ($selected)
            return "#FFFFFFE0";
        else
            return "#FFAFAFA0";
    }

    protected static function do_get_dimmed_item_text_color($selected)
    {
        if ($selected)
            return "#FFAFAFA0";
        else
            return "#FF808080";
    }

    ///////////////////////////////////////////////////////////////////////

    protected static function do_get_button_color($selected)
    {
        if ($selected)
            return "#FFFFFFE0";
        else
            return "#FFAFAFA0";
    }

    protected function do_get_back_img_geom($sel_state)
    {
        $dx = $sel_state->slideshow ?
            self::$LEFT_BG_WIDTH * self::$DEFAULT_TRANSITION_STEPS : 0;
        return GCompGeom::place_top_left(
            L::$VOD_BACK_ARROW_SIZE,
            L::$VOD_BACK_ARROW_SIZE,
            L::$VOD_BACK_ARROW_LEFT2 - $dx,
            L::$VOD_PANEL_TOP + (68 - L::$VOD_BACK_ARROW_SIZE) / 2);
    }

    protected function do_get_poster_panel_geom($sel_state)
    {
        $dx = $sel_state->slideshow ? self::$LEFT_BG_WIDTH : 0;
        return GCompGeom::place_top_left(
            L::$VOD_POSTER_WIDTH, -1,
            L::$SCREEN_LEFT - $dx,
            L::$VOD_PANEL_TOP);
    }

    protected function do_get_left_bg_geom($sel_state)
    {
        $dx = $sel_state->slideshow ? self::$LEFT_BG_WIDTH : 0;
        return GCompGeom::place_top_left(
            self::$LEFT_BG_WIDTH, L::$HEIGHT, -$dx, 0);
    }

    protected function do_get_left_bg_def($sel_state)
    {
        $def = GCompsFactory::get_rect_def(
            $this->do_get_left_bg_geom($sel_state),
            null, self::$DARKEN_COLOR);
        $def[GComponentDef::id] = 'left_bg';
        return $def;
    }

    protected function do_get_center_bg_def($sel_state)
    {
        $panel_w = L::$WIDTH - L::$SCREEN_RIGHT -
            L::$SCREEN_LEFT - L::$VOD_POSTER_WIDTH - self::$HGAP;

        $def = GCompsFactory::get_rect_def(
            $this->do_get_center_bg_geom($sel_state),
            null, self::$DARKEN_COLOR);
        $def[GComponentDef::id] = "center_bg";
        return $def;
    }

    ///////////////////////////////////////////////////////////////////////

    protected function do_get_sel_geom($sel_state)
    {
        if ($sel_state->sel_kind == 0)
        {
            return GCompGeom::place_same_size(
                10, -2, -10, 5, "btn_$sel_state->button_id");
        }
        else
        {
            $h_extent =
                $this->is_desc_button_id($sel_state->button_id) ? 5 : 0;
            return GCompGeom::place_same_size(
                $h_extent * 2, 16, -$h_extent, -8, "item$sel_state->item_sel");
        }
    }

    ///////////////////////////////////////////////////////////////////////

    protected static function do_get_selection_state($slideshow,
        $sel_kind, $button_id, $item_sel, $item_top, $item_id)
    {
        return (object) array(
            'slideshow' => $slideshow,
            'sel_kind' => $sel_kind,
            'button_id' => $button_id,
            'item_sel' => $item_sel,
            'item_top' => $item_top,
            'item_id' => $item_id);
    }

    public function get_selection_state()
    {
        return self::do_get_selection_state(
            $this->cur_slideshow,
            $this->cur_sel_kind,
            $this->cur_button_id,
            $this->cur_item_sel,
            $this->cur_item_top,
            $this->cur_item_id);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array(
            GUI_EVENT_KEY_ENTER => $this->custom_action('enter'),

            GUI_EVENT_KEY_LEFT => $this->change_action('left'),
            GUI_EVENT_KEY_P_PLUS => $this->change_action('page_up'),
            GUI_EVENT_KEY_P_MINUS => $this->change_action('page_down'),
            GUI_EVENT_KEY_UP => $this->change_action('up'),
            GUI_EVENT_KEY_DOWN => $this->change_action('down'),
            GUI_EVENT_KEY_FIP_PREV => $this->change_action('up'),
            GUI_EVENT_KEY_FIP_NEXT => $this->change_action('down'),
            GUI_EVENT_KEY_RIGHT => $this->change_action('right'),

// Uncomment for specific manual testing.
//            GUI_EVENT_KEY_INFO => ActionFactory::invalidate_all_folders(),
        );
    }

    ///////////////////////////////////////////////////////////////////////

    private static function add_def(&$defs, $def)
    {
        if (isset($def))
            $defs[] = $def;
    }

    ///////////////////////////////////////////////////////////////////////

    protected function do_get_poster_panel_def($sel_state)
    {
        $inf = $this->do_get_poster_info($sel_state);

        $defs = array();
        $y = 0;

        if (isset($inf->title))
        {
            $title_def = GCompsFactory::get_ttf_label_def(
                GCompGeom::place_top_center(
                    -1, L::$VOD_BUTTON_HEIGHT + L::$PAR_VGAP_SMALL * 2,
                    0, -L::$PAR_VGAP_SMALL),
                null, $inf->title, 1, "#FFFFFFE0", L::$BIG_TEXT_SIZE);
            $y += L::$VOD_BUTTON_HEIGHT + L::$VOD_POSTER_BOTTOM - L::$PAR_VGAP_SMALL;
            $defs[] = $title_def;
        }
        else if (isset($inf->logo_url))
        {
            $title_def = GCompsFactory::get_image_def(
                GCompGeom::place_top_center(
                    $inf->logo_width, $inf->logo_height,
                    0, 0),
                null, $inf->logo_url, false, true);
            $y += $inf->logo_height + L::$VOD_POSTER_TOP;
            $defs[] = $title_def;
        }

        $img_def = GCompsFactory::get_image_def(
            GCompGeom::place_top_left(
                $inf->width, $inf->height, 0, 0),
            null, $inf->poster_url, true, true,
            $inf->not_loaded_url, $inf->load_failed_url, $inf->low_quality_url);
        $defs[] = GCompsFactory::get_ppane_def(null,
            GCompGeom::place_top_left(
                $inf->width, $inf->height, 0, $y),
            array($img_def));

        $button_ids = $this->get_cur_button_ids();

        $y += $inf->height + L::$VOD_POSTER_BOTTOM - L::$TEXT_BOTTOM;
        foreach ($button_ids as $button_id)
        {
            $selected = $button_id == $this->cur_button_id;
            $caption = $this->get_button_caption_by_id($button_id);

            $h = L::$TEXT_SIZE + L::$TEXT_BOTTOM * 2;
            $button_def = GCompsFactory::get_ttf_label_def(
                GCompGeom::place_top_left(
                    $inf->width - L::$VOD_BUTTON_TEXT_LEFT,
                    $h,
                    L::$VOD_BUTTON_TEXT_LEFT, 
                    $y + (L::$VOD_BUTTON_HEIGHT - $h) / 2),
                null, $caption, 1,
                self::do_get_button_color($selected), 
                L::$TEXT_SIZE);
            $button_def[GComponentDef::id] = "btn_$button_id";
            $defs[] = $button_def;

            $y += L::$VOD_BUTTON_HEIGHT;
        }

        return GCompsFactory::get_panel_def("poster_pane",
            $this->do_get_poster_panel_geom($sel_state),
            null,
            $defs);
    }

    private function do_get_right_panel_def($sel_state)
    {
        return GCompsFactory::get_panel_def("right_panel",
            GCompGeom::place_top_left(
                self::$RIGHT_PANEL_WIDTH,
                L::$HEIGHT - L::$VOD_PANEL_TOP - L::$VOD_PANEL_BOTTOM,
                self::$RIGHT_PANEL_X,
                L::$VOD_PANEL_TOP),
            NULL,
            $this->do_get_right_panel_children($sel_state));
    }

    protected function do_get_hints_pane_geom($sel_state)
    {
        $h = L::$VOD_BUTTON_HEIGHT + L::$COLOR_ICON_TEXT_DY;
        if ($this->is_first_button_id($sel_state->button_id))
        {
            return GCompGeom::place_top_left(
                $this->get_right_panel_width(), $h,
                self::$RIGHT_PANEL_X, L::$HEIGHT);
        }
        $w = $this->is_desc_button_id($sel_state->button_id) ?
            $this->get_right_panel_width() : -1;
        return GCompGeom::place_top_left($w, $h,
            self::$RIGHT_PANEL_X, L::$HEIGHT - L::$HINTS_BOTTOM_OFFSET);
    }

    protected function do_get_desc_hints_pane_children(&$defs,
        $geom, $sel_state)
    {
        $is_desc_button =
            $this->is_first_button_id($sel_state->button_id) ||
            $this->is_desc_button_id($sel_state->button_id);
        if ($is_desc_button)
        {
            $defs[] = GCompsFactory::get_image_def($geom, null,
                "plugin_file://icons/7.png");

            $defs[] = GCompsFactory::get_ttf_label_def(
                GCompGeom::place_next_right_align_top(
                    -1, L::$VOD_BUTTON_HEIGHT,
                    0, -L::$COLOR_ICON_TEXT_DY),
                GCompMargins::all(L::$TEXT_ICON_HGAP, 0, L::$HINTS_SMALL_HGAP, 0),
                T::t("action_scroll"), 
                1, "#FFAFAFA0", L::$TEXT_SIZE);

            $img_def = GCompsFactory::get_image_def(
                GCompGeom::place_top_right(
                    L::$UP_DOWN_WIDTH, L::$UP_DOWN_HEIGHT),
                null,
                "gui_skin://special_icons/up_down.png");
            $img_def[GComponentDef::id] = "item0";

            $defs[] = $img_def;
        }
    }

    protected function do_get_hints_pane($sel_state)
    {
        $geom = $this->do_get_hints_pane_geom($sel_state);
        $children = $this->do_get_hints_pane_children($sel_state);
        if (!isset($geom) || !isset($children))
            return null;
        return GCompsFactory::get_panel_def("hints_pane",
            $geom, null, $children);
    }

    private function do_get_comp_defs($sel_state)
    {
        $defs = array();

        $with_darkened_bg = $this->with_darkened_bg();
        if ($with_darkened_bg)
        {
            self::add_def($defs,
                $this->do_get_left_bg_def($sel_state));
            self::add_def($defs,
                $this->do_get_center_bg_def($sel_state));
        }

        self::add_def($defs,
            $this->do_get_top_pane_def($sel_state));

        $back_def = GCompsFactory::get_image_def(
            $this->do_get_back_img_geom($sel_state),
            null, "plugin_file://icons/7.png");
        $back_def[GComponentDef::id] = "back_img";
        self::add_def($defs, $back_def);

        $sel_def = GCompsFactory::get_image_def(
            $this->do_get_sel_geom($sel_state), null,
            "plugin_file://icons/pic3.aai", false, true);
        $sel_def[GComponentDef::id] = "sel";
        self::add_def($defs, $sel_def);

        self::add_def($defs,
            $this->do_get_poster_panel_def($sel_state));
        self::add_def($defs,
            $this->do_get_right_panel_def($sel_state));

        self::add_def($defs,
            $this->do_get_hints_pane($sel_state));

        return $defs;
    }

    public function get_window_def(MediaURL $media_url, &$plugin_cookies)
    {
        CurrentMovieFiles::get_instance()->sync_with_media_url($media_url);
        CurrentMovieFiles::get_instance()->ensure_non_dirty();

        $this->sync_current_folder($media_url, $this->cur_sel_state);

        $this->set_cur_object_visited();

        $bg = $this->get_window_bg_info();
        $sel_state = $this->get_selection_state();

        $fip_text = $this->get_fip_text();

        $opaque_background = isset($bg->url) &&
            isset($bg->opaque) && $bg->opaque;

        return GCompsFactory::get_window_def(
            $this->do_get_comp_defs($sel_state),
			null,//!!!!!!!!!!!!!!!!!!!!!!!!!!!
            $bg->url,
            $bg->color,
            $bg->not_loaded_url,
            $bg->async_loading,
            $bg->playback_alpha,
            isset($bg->fit_def) ? $bg->fit_def : null,
            $fip_text, $opaque_background);
    }

    ///////////////////////////////////////////////////////////////////////

    protected function do_get_add_favorite_action(
        $fav_media_url, $caption, $icon_url)
    {
        return ActionFactory::add_to_favorites(
            $fav_media_url, $caption, $icon_url, true,
            UserInputHandlerRegistry::create_action($this,
                'post_add_to_favorites'));
    }

    protected function do_get_remove_favorite_action($id)
    {
        return ActionFactory::remove_from_favorites($id, 
            UserInputHandlerRegistry::create_action($this,
                'post_remove_from_favorites'));
    }

    private function get_change_behaviour(&$media_url, &$plugin_cookies,
        $post_action=null)
    {
        return ActionFactory::change_behaviour(
            $this->get_action_map($media_url, $plugin_cookies),
            $this->get_timer($media_url, $plugin_cookies),
            $post_action);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print($this->get_id() . ' screen: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $with_darkened_bg = $this->with_darkened_bg();

        $this->prepare_handle_user_input($user_input);

        $media_url = MediaURL::decode($user_input->parent_media_url);

        CurrentMovieFiles::get_instance()->sync_with_media_url($media_url);
        $stack_updated =
            $this->sync_current_folder($media_url, $this->cur_sel_state);

        hd_print("cur_sel_state: ".$this->get_cur_sel_state_str());
        hd_print("cur_sel_state2: ".json_encode($this->get_selection_state()));

        if (!isset($user_input->control_id))
            return null;

        $no_animation = isset($user_input->no_animation) &&
            $user_input->no_animation == 1;

        $op = $user_input->control_id == 'change' ? $user_input->op : 'none';
        if ($user_input->control_id == 'enter')
            $op = 'right';

        $button_id = $this->cur_button_id;
        $num_buttons = count($this->get_cur_button_ids());
        if ($num_buttons == 0)
            throw new Exception("Illegal state (ent:01)");

        $num_items = count($this->cur_object_ids);

        $enter_required = $op == 'right' &&
            ($this->cur_sel_kind == 1 || $num_items == 0);

        if ($enter_required)
        {
            if (!$this->cur_slideshow)
                return $this->handle_enter($media_url, $plugin_cookies);
            $op = 'down';
        }

        if ($user_input->control_id == 'timer')
        {
            $async_changed = false;
            if ($op == 'none' && !$stack_updated)
                $async_changed = $this->check_async_loader();

            if ($async_changed)
            {
                return ActionFactory::invalidate_folders(
                    array($media_url->get_raw_string()));
            }
            return $this->get_change_behaviour($media_url, $plugin_cookies);
        }

        if ($user_input->control_id == 'post_add_to_favorites' ||
            $user_input->control_id == 'post_remove_from_favorites')
        {
            $add = $user_input->control_id == 'post_add_to_favorites';
            $inv_action = ActionFactory::invalidate_folders(
                array($media_url->get_raw_string()));
            return Favorites::get_update_status_dialog($this->cur_type,
                $add, $user_input->action_status, $inv_action);
        }

        $change_behaviour = false;

        $res = $this->handle_other_action($user_input, $media_url, $plugin_cookies);
        $change_post_action = null;
        if (isset($res))
        {
            if (isset($res->action) && !isset($res->op))
                return $res->action;
            $change_behaviour = $res->change_behaviour;
            if (isset($res->action))
                $change_post_action = $res->action;
            if (isset($res->op))
                $op = $res->op;
        }

        $change_defs = null;

        $slideshow_change = false;
        $scroll_change = false;
        if ($op != 'none')
        {
            if ($op == 'left' && $this->cur_sel_kind == 0)
                return ActionFactory::close_and_run();

            $button_sel = $this->get_button_sel_by_id($button_id);
            $new_button_sel = $button_sel;

            $new_slideshow = $this->cur_slideshow;
            $new_sel_kind = $this->cur_sel_kind;
            $new_button_id = $this->cur_button_id;
            $new_item_sel = $this->cur_item_sel;
            $new_item_top = $this->cur_item_top;
            
            $y_offset = 0;
            if ($this->cur_slideshow == 1)
                $new_slideshow = 0;
            else
            {
                $with_scroll = $this->is_scroll_panel_content(
                    $this->cur_button_id);
                if ($with_scroll)
                {
                    if ($op == 'page_up')
                        $y_offset = -L::$VOD_SCROLL_SIZE;
                    else if ($op == 'page_down')
                        $y_offset = L::$VOD_SCROLL_SIZE;
                }

                if ($this->cur_sel_kind == 1 &&
                    ($op == 'page_up' || $op == 'page_down'))
                {
                    $page_size =
                        MainStack::get_item_page_size($button_id);
                    if ($op == 'page_up')
                    {
                        $new_item_sel = max(0,
                            $this->cur_item_sel - $page_size);
                    }
                    else
                    {
                        $new_item_sel = min($num_items - 1,
                            $this->cur_item_sel + $page_size);
                    }
                    $new_item_top = MainStack::get_item_top(
                        $new_item_sel, $num_items, $button_id);
                }

                if ($op == 'right')
                {
                    if ($this->cur_sel_kind != 0)
                        return null;
                    if ($num_items == 0)
                        return null;

                    $new_sel_kind = 1;
                }
                else if ($op == 'left')
                {
                    $new_sel_kind--;
                }
                else if ($op == 'force_slideshow')
                {
                    if ($this->cur_sel_kind == 0 && $button_sel == 0)
                        $new_slideshow = 1;
                }
                else if ($op == 'up')
                {
                    if ($this->cur_sel_kind == 0)
                    {
                        if ($button_sel == 0)
                        {
                            if ($this->is_slideshow_enabled($user_input))
                                $new_slideshow = 1;
                        }
                        else
                            $new_button_sel = $button_sel - 1;
                    }
                    else if ($with_scroll && $num_items == 1)
                        $y_offset = -L::$VOD_ARROWS_SCROLL_SIZE;
                    else
                    {
                        $new_item_sel = max(0, $this->cur_item_sel - 1);
                        $new_item_top = MainStack::get_item_top(
                            $new_item_sel, $num_items, $button_id);
                    }
                }
                else if ($op == 'down')
                {
                    if ($this->cur_sel_kind == 0)
                        $new_button_sel = min($num_buttons - 1, $button_sel + 1);
                    else if ($with_scroll && $num_items == 1)
                        $y_offset = L::$VOD_ARROWS_SCROLL_SIZE;
                    else
                    {
                        $new_item_sel = min($num_items - 1,
                            $this->cur_item_sel + 1);
                        $new_item_top = MainStack::get_item_top(
                            $new_item_sel, $num_items, $button_id);
                    }
                }
            }

            if ($new_button_sel != $button_sel)
            {
                $new_item_sel = 0;
                $new_item_top = 0;
                $new_button_id =
                    $this->get_button_id_by_sel($new_button_sel);
            }

            if ($y_offset != 0)
            {
                $scroll_change = true;
                $change_defs[] = GCompsFactory::get_change_def(
                    'center_spane', null, null,
                    GCompsFactory::get_view_position_def(0, $y_offset));
            }

            $new_item_id = '';
            if ($new_item_sel >= 0 && $new_item_sel < $num_items)
                $new_item_id = $this->cur_object_ids[$new_item_sel];
            $new_sel_state = self::do_get_selection_state(
                $new_slideshow, $new_sel_kind, $new_button_id,
                $new_item_sel, $new_item_top, $new_item_id);

            $slideshow_change = $new_slideshow != $this->cur_slideshow;

            if ($new_slideshow != $this->cur_slideshow ||
                $new_sel_kind != $this->cur_sel_kind ||
                $new_button_sel != $button_sel ||
                $new_item_sel != $this->cur_item_sel)
            {
                $def_transition = $no_animation ?
                    GCOMP_TRANSITION_NONE : GCOMP_TRANSITION_DEFAULT;
                $transition = $def_transition;
                if ($new_button_sel == $button_sel &&
                    $new_item_top != $this->cur_item_top)
                {
                    $transition = GCOMP_TRANSITION_NONE;
                }
                $move_transition = $def_transition;

                $change_defs[] = GCompsFactory::get_change_def('sel',
                    $this->do_get_sel_geom($new_sel_state),
                    null, null, null, null, $transition);

                if ($new_slideshow != $this->cur_slideshow)
                {
                    if ($with_darkened_bg)
                    {
                        $change_defs[] = GCompsFactory::get_change_def('left_bg',
                            $this->do_get_left_bg_geom($new_sel_state),
                            null, null, null, null, $transition);

                        $change_defs[] = GCompsFactory::get_change_def('center_bg',
                            $this->do_get_center_bg_geom($new_sel_state),
                            null, null, null, null, $transition);
                    }

                    $change_defs[] = GCompsFactory::get_change_def('back_img',
                        $this->do_get_back_img_geom($new_sel_state),
                        null, null, null, null, $transition);

                    $change_defs[] = GCompsFactory::get_change_def('poster_pane',
                        $this->do_get_poster_panel_geom($new_sel_state),
                        null, null, null, null, $transition);

                    $change_defs[] = GCompsFactory::get_change_def(
                        "center_pane",
                        $this->do_get_center_geom($new_sel_state),
                        null, null, null, null, $transition);

                    $geom = $this->do_get_hints_pane_geom($new_sel_state);
                    if (isset($geom))
                    {
                        $change_defs[] = GCompsFactory::get_change_def(
                            "hints_pane", $geom,
                            null, null, null, null, $transition);
                    }

                    if (isset($user_input->play_mode) &&
                        $user_input->play_mode != 'none' &&
                        $new_slideshow == 1)
                    {
                        $change_post_action = ActionFactory::switch_osd_mode(
                            true, $change_post_action);
                    }
                }
                else if ($new_button_sel != $button_sel)
                {
                    $this->apply_button_change($new_button_id);

                    $change_defs[] = GCompsFactory::get_change_def(
                        "btn_$button_id", null,
                        array('color' => self::do_get_button_color(false)),
                        null, null, null, $def_transition);
                    $change_defs[] = GCompsFactory::get_change_def(
                        "btn_$new_sel_state->button_id", null,
                        array('color' => self::do_get_button_color(true)),
                        null, null, null, $def_transition);

                    $part_darkened = $this->is_partially_darkened(
                        $this->cur_sel_state);
                    $new_part_darkened = $this->is_partially_darkened(
                        $new_sel_state);
                    $move = $part_darkened || $new_part_darkened;
                    $fade_transition = GCOMP_TRANSITION_NONE;
//                    $fade_transition = $move ?
//                        GCOMP_TRANSITION_NONE : GCOMP_TRANSITION_FADE;

                    $geom = $this->do_get_top_pane_geom($new_sel_state);
                    if (isset($geom))
                    {
                        $change_defs[] = GCompsFactory::get_change_def(
                            "top_pane", $geom,
                            null, null, null, null,
                            $move ? $move_transition : GCOMP_TRANSITION_NONE);
                    }

                    if ($with_darkened_bg)
                    {
                        $change_defs[] = GCompsFactory::get_change_def(
                            "center_bg",
                            $this->do_get_center_bg_geom($new_sel_state),
                            null, null, null, null,
                            $move ? $move_transition : GCOMP_TRANSITION_NONE);
                    }

                    if ($move)
                    {
                        $change_defs[] = GCompsFactory::get_change_def(
                            "center_pane",
                            $this->do_get_center_geom($new_sel_state),
                            null, null, null, null, $move_transition);

                        $geom = $this->do_get_hints_pane_geom($new_sel_state);
                        if (isset($geom))
                        {
                            $change_defs[] = GCompsFactory::get_change_def(
                                "hints_pane", $geom,
                                null, null, null, null, $move_transition);
                        }

                        if ($new_part_darkened)
                        {
                            $change_defs[] = GCompsFactory::get_change_def(
                                'center_spane', null, null,
                                GCompsFactory::get_view_position_def(0, -10000),
                                null, null, $move_transition);
                        }
                    }
                    else
                    {
                        $change_defs[] = GCompsFactory::get_change_def('right_panel',
                            null, null, null, null,
                            $this->do_get_right_panel_children($new_sel_state),
                            $fade_transition);

                        $children = $this->do_get_hints_pane_children($new_sel_state);
                        if (isset($children))
                        {
                            $change_defs[] = GCompsFactory::get_change_def("hints_pane",
                                null, null, null, null,
                                $children, GCOMP_TRANSITION_NONE);
                        }
                    }
                }
                else if ($new_item_sel != $this->cur_item_sel)
                {
                    $page_size = MainStack::get_item_page_size($button_id);
                    $align_bottom_num_seps =
                        $this->get_align_bottom_num_seps(
                            $this->cur_item_top,
                            $this->cur_item_sel,
                            $num_items, $page_size);
                    $new_align_bottom_num_seps =
                        $this->get_align_bottom_num_seps(
                            $new_item_top, $new_item_sel,
                            $num_items, $page_size);

                    if ($new_item_top != $this->cur_item_top ||
                        $new_align_bottom_num_seps != $align_bottom_num_seps)
                    {
                        $from = 0;
                        $to = $num_items;

                        for ($i = $from; $i < $to; $i++)
                        {
                            $new_geom = $this->do_get_item_geom(
                                $new_sel_state, $i - $new_item_top,
                                $new_align_bottom_num_seps);
                            $change_defs[] = GCompsFactory::get_change_def(
                                "item".$i, $new_geom,
                                null, null, null, null, $transition);
                        }

                        foreach ($this->cur_object_list_seps as $sep_ndx => $sep_msg)
                        {
                            $change_defs[] = GCompsFactory::get_change_def(
                                "sep".$sep_ndx,
                                $this->do_get_item_sep_geom($new_sel_state,
                                    $sep_ndx - $new_item_top,
                                    $new_align_bottom_num_seps),
                                null, null, null, null, $transition);
                        }
                    }

                    $change_defs[] = GCompsFactory::get_change_def(
                        "img".$this->cur_item_sel,
                        $this->do_get_item_img_geom(false, $this->cur_sel_state),
                        null, null, null, null, $transition);
                    $change_defs[] = GCompsFactory::get_change_def(
                        "img".$new_item_sel,
                        $this->do_get_item_img_geom(true, $new_sel_state),
                        null, null, null, null, $transition);

                    $change_defs[] = GCompsFactory::get_change_def(
                        "name".$this->cur_item_sel, null,
                        array(
                            'text_size' => $this->do_get_item_text_size(false, $this->cur_sel_state),
                            'color' => self::do_get_item_text_color(false),
                        ),
                        null, null, null, $transition);
                    $change_defs[] = GCompsFactory::get_change_def(
                        "name".$new_item_sel, null,
                        array(
                            'text_size' => $this->do_get_item_text_size(true, $new_sel_state),
                            'color' => self::do_get_item_text_color(true),
                        ),
                        null, null, null, $transition);

                    $change_defs[] = GCompsFactory::get_change_def(
                        "2nd_name".$this->cur_item_sel, null,
                        array(
                            'text_size' => $this->do_get_item_text_size(false, $this->cur_sel_state),
                            'color' => $this->do_get_item_text_color2(
                                false, $this->cur_item_sel),
                        ),
                        null, null, null, $transition);
                    $change_defs[] = GCompsFactory::get_change_def(
                        "2nd_name".$new_item_sel, null,
                        array(
                            'text_size' => $this->do_get_item_text_size(true, $new_sel_state),
                            'color' => $this->do_get_item_text_color2(
                                true, $new_item_sel),
                        ),
                        null, null, null, $transition);

                    $change_defs[] = GCompsFactory::get_change_def(
                        "3rd_name".$this->cur_item_sel, null,
                        array(
                            'text_size' => $this->do_get_item_text_size(false, $this->cur_sel_state),
                            'color' => self::do_get_dimmed_item_text_color(false),
                        ),
                        null, null, null, $transition);
                    $change_defs[] = GCompsFactory::get_change_def(
                        "3rd_name".$new_item_sel, null,
                        array(
                            'text_size' => $this->do_get_item_text_size(true, $new_sel_state),
                            'color' => self::do_get_dimmed_item_text_color(true),
                        ),
                        null, null, null, $transition);

                    $object_info_panel_children =
                        $this->do_get_object_info_panel_children($new_sel_state);
                    if (isset($object_info_panel_children))
                    {
                        $change_defs[] = GCompsFactory::get_change_def('object_info_panel',
                            null, null, null, null,
                            $object_info_panel_children,
                            GCOMP_TRANSITION_NONE);
                    }
                }

                $this->cur_slideshow = $new_slideshow;
                $this->cur_sel_kind = $new_sel_kind;
                $this->cur_item_sel = $new_item_sel;
                $this->cur_item_top = $new_item_top;
                $this->cur_button_id = $new_button_id;
                $this->adjust_cur_item_id();
            }
        }

        $change_action = null;
        if (isset($change_defs))
        {
            $this->cur_sel_state = $this->get_selection_state();

            $num_steps = $slideshow_change ?
                self::$DEFAULT_TRANSITION_STEPS :
                ($scroll_change ?
                 self::$DEFAULT_SCROLL_STEPS :
                 self::$DEFAULT_TRANSITION_STEPS);
            $fip_text = $this->get_fip_text();
            $change_action = ActionFactory::change_gcomps($this,
                $change_defs, false, $num_steps, $fip_text, $change_post_action);
        }
        else
            $change_action = $change_post_action;

        if (!$change_behaviour)
            return $change_action;

        return $this->get_change_behaviour(
            $media_url, $plugin_cookies, $change_action);
    }
}

?>
