<?php

class ShortMovie
{
    public $id;
    public $name;
    public $poster_url;

    public function __construct($id, $name, $poster_url){
        if (is_null($id))
            throw new Exception("ShortMovie::id is null");

        $this->id = strval($id);
        $this->name = strval($name);
        $this->poster_url = strval($poster_url);
    }
}

class ShortMovieRange
{
    public $from_ndx;
    public $total;
    public $short_movies;

    public function __construct($from_ndx, $total, $short_movies = null){
        $this->from_ndx = intval($from_ndx);
        $this->total = intval($total);
        $this->short_movies = $short_movies === null ?
            array() : $short_movies;
    }
}

class MovieSeries
{
    public $id;

    public function __construct($id){
        if (is_null($id))
            throw new Exception("MovieSeries::id is null");

        $this->id = strval($id);
    }

    public $name = '';
    public $playback_url = '';
    public $playback_url_is_stream_url = true;
	public $file_info = array();
}

class Movie implements UserInputHandler
{
	const ID = 'smart_movie';
	public $ims;
	
    public $id;
    public $name = '';
    public $name_original = '';
    public $description = '';
    public $poster_url = '';
    public $length_min = -1;
    public $year = 0;
    public $directors_str = '';
    public $scenarios_str = '';
    public $actors_str = '';
    public $genres_str = '';
    public $rate_imdb = '';
    public $rate_kinopoisk = '';
    public $rate_mpaa = '';
    public $country = '';
    public $budget = '';
	public $rate_details;
    public $details;
    public $series_list = null;
	
	public $videoInfoShow = false;
	public $sleepAfter = false;
	public $sleepAlert = true;
	public $delayTimer = false;
	static $timeInfoShow = false;
	
    public function __construct($id){
        if (is_null($id))
            throw new Exception("Movie::id is null");

        $this->id = strval($id);
		UserInputHandlerRegistry::get_instance()->register_handler($this);
    }
	
	public function get_handler_id(){ return self::ID; }
	
    private function to_string($v){
        return $v === null ? '' : strval($v);
    }

    private function to_int($v, $default_value){
        $v = strval($v);
        if (!is_numeric($v))
            return $default_value;
        $v = intval($v);
        return $v <= 0 ? $default_value : $v;
    }

    public function set_data(
        $name,
        $name_original,
        $description,
        $poster_url,
        $length_min,
        $year,
        $directors_str,
        $scenarios_str,
        $actors_str,
        $genres_str,
        $rate_imdb,
        $rate_kinopoisk,
        $rate_mpaa,
        $country,
        $budget,
		$details,
		$rate_details
	){
        $this->name = $this->to_string($name);
        $this->name_original = $this->to_string($name_original);
        $this->description = $this->to_string($description);
        $this->poster_url = $this->to_string($poster_url);
        $this->length_min = $this->to_int($length_min, -1);
        $this->year = $this->to_int($year, -1);
        $this->directors_str = $this->to_string($directors_str);
        $this->scenarios_str = $this->to_string($scenarios_str);
        $this->actors_str = $this->to_string($actors_str);
        $this->genres_str = $this->to_string($genres_str);
        $this->rate_imdb = $this->to_string($rate_imdb);
        $this->rate_kinopoisk = $this->to_string($rate_kinopoisk);
        $this->rate_mpaa = $this->to_string($rate_mpaa);
        $this->country = $this->to_string($country);
        $this->budget = $this->to_string($budget);        
		$this->details = $details;
		$this->rate_details = $rate_details;
        $this->series_list = array();
    }

    public function add_series_data(
		$id, 
		$name,
        $playback_url, 
		$playback_url_is_stream_url,
		$file_info
	){
        $series = new MovieSeries($id);
		$series->id = $this->to_string($id);
        $series->name = $this->to_string($name);
        $series->playback_url = $this->to_string($playback_url);
        $series->playback_url_is_stream_url = $playback_url_is_stream_url;
		$series->file_info = $file_info;
		
        $this->series_list[] = $series;
    }

    public function get_movie_array(){
        return array(
            PluginMovie::name => $this->name,
            PluginMovie::name_original => $this->name_original,
            PluginMovie::description => $this->description,
            PluginMovie::poster_url => $this->poster_url,
            PluginMovie::length_min => $this->length_min,
            PluginMovie::year => $this->year,
            PluginMovie::directors_str => $this->directors_str,
            PluginMovie::scenarios_str => $this->scenarios_str,
            PluginMovie::actors_str => $this->actors_str,
            PluginMovie::genres_str => $this->genres_str,
            PluginMovie::rate_imdb => $this->rate_imdb,
            PluginMovie::rate_kinopoisk => $this->rate_kinopoisk,
            PluginMovie::rate_mpaa => $this->rate_mpaa,
            PluginMovie::country => $this->country,
            PluginMovie::budget => $this->budget,
			PluginMovie::details =>  $this->details,
			PluginMovie::rate_details   => $this->rate_details,
        );
    }

    public function get_vod_info($sel_id, $buffering_ms){
        if (!is_array($this->series_list) ||
            count($this->series_list) == 0){
            throw new Exception('Invalid movie: series list is empty');
        }
		$timeInfoShowRendSettings = HD::getItemsArr('timeInfoShowRendSettings', true);
		$clock = isset($timeInfoShowRendSettings['clock']) ? $timeInfoShowRendSettings['clock'] : 0;
		if ($clock > 1){
			if (self::$timeInfoShow == false){
				if ($timeInfoShowRendSettings['position'] == 4)
					$timeInfoShowRendSettings['position'] = rand (0,3);
				self::$timeInfoShow = $timeInfoShowRendSettings;
			}
		}
        $initial_position_ms = $series_array = array();
        $initial_series_ndx = -1;
		$viewedItemsArr = HD::getItemsArr('viewedItemsArr');
		$viewedItemsTmpArr = HD::getItemsTmpArr('viewedItemsTmpArr');
		if (count($viewedItemsTmpArr) > 0){
			foreach ($viewedItemsTmpArr as $k => $v){
				if (isset($viewedItemsArr[$k]))
					$viewedItemsArr[$k] = array_merge( $viewedItemsArr[$k], $v);
				else
					$viewedItemsArr[$k] = $v;
			}
		}
		$description = $this->description;
		$poster_url = $this->poster_url;
        foreach ($this->series_list as $ndx => $series){
			if ($series->playback_url == 'dummy')
				continue;
            if (!is_null($sel_id) && $series->id === $sel_id){
                $initial_series_ndx = $ndx;
				if ($this->series_list[$ndx]->file_info['class_name'] == 'TransmissionUPnP')
					if($this->series_list[$ndx]->file_info['percent'] != 100)
						TransmissionUPnP::setTransmissionActionPriority($this->series_list[$ndx]->file_info['torrent_id'], $this->series_list[$ndx]->file_info['file_id']);
			}
			$tt = '';
			$this->ims = 0;
			if (isset($viewedItemsArr[$series->id])){
				if ($viewedItemsArr[$series->id] == 'viewed')
					$tt = '[ПРОСМ] ';
				else{
					$tt = '[' . HD::sec_format_duration($viewedItemsArr[$series->id]['p']) . '/' . HD::sec_format_duration($viewedItemsArr[$series->id]['d']) . '] ';
					$pos = intval($viewedItemsArr[$series->id]['p']);
					if ($pos < 0)
						$pos = 0;
					if ($pos + 5 >= intval($viewedItemsArr[$series->id]['d']))
						$pos = 0;
					$initial_position_ms[$ndx] = $pos * 1000;
				}
			}
			
            $series_array[] = array(
                PluginVodSeriesInfo::name => $tt . $series->name,
                PluginVodSeriesInfo::playback_url => $series->playback_url,
                PluginVodSeriesInfo::playback_url_is_stream_url => $series->playback_url_is_stream_url,
            );
        }
		if (isset($this->series_list[$initial_series_ndx]->file_info['description']))
			$description = $this->series_list[$initial_series_ndx]->file_info['description'];
		if (isset($this->series_list[$initial_series_ndx]->file_info['poster_url']))
			$poster_url = $this->series_list[$initial_series_ndx]->file_info['poster_url'];
		
		if (isset($initial_position_ms[$initial_series_ndx]))
			$this->ims = $initial_position_ms[$initial_series_ndx];
        return array(
            PluginVodInfo::id => $this->id,
            PluginVodInfo::name => $this->name,
            PluginVodInfo::description => $description,
            PluginVodInfo::poster_url => $poster_url,
            PluginVodInfo::series => $series_array,
            PluginVodInfo::initial_series_ndx => $initial_series_ndx,
            PluginVodInfo::buffering_ms => $buffering_ms,
			PluginVodInfo::actions => $this->get_movie_actions(),
			PluginVodInfo::timer => array(GuiTimerDef::delay_ms => 5000),
			PluginVodInfo::initial_position_ms => $this->ims,
        );
    }
	
	public function handle_user_input(&$user_input, &$plugin_cookies){
		if ($user_input->control_id == 'classAction'){
			$defs = array();
			hd_silence_warnings();
			if (isset($user_input->indiClass)){
				$class_name = $user_input->indiClass;
				return $class_name::classAction ($defs, $user_input);
			}
			return SmartConfig::classAction ($defs, $user_input);
		}
		if ($user_input->control_id == 'event_timer'){
			$actions = $this->get_movie_actions();
			$post_action = null;
			$series_id = $this->series_list[$user_input->plugin_vod_series_ndx]->id;
			if (!isset($this->playback_info[$series_id])){
				$itemsHistory = HD::getItemsArr('itemsHistory');
				if (isset($itemsHistory[$this->history_mUrl]))
					unset ($itemsHistory[$this->history_mUrl]);
				$history_str = "||Просмотрено: ".date("d.m.Y") .'|'. $this->series_list[$user_input->plugin_vod_series_ndx]->name . $this->series_list[$user_input->plugin_vod_series_ndx]->file_info['detailed_info'];
				$itemsHistory = array($this->history_mUrl => $history_str) + $itemsHistory;
				$posterCasheHist = (HD::getItemStr('posterCasheHist') != '') ? HD::getItemStr('posterCasheHist') : 0;
				if ($posterCasheHist == 1){
					$output = array_slice($itemsHistory, 310);
					$cachePath = HD::get_data_path() . "/pluginCache/";
					if (count($output) > 0){
						foreach ($output as $k => $v){
							$cUrl = MediaURL::decode($k);
							$arrCachePath = $cachePath . md5($cUrl->movie_id . $cUrl->class_name) . '.srz';
							$cachePosterUrl = $cachePath . md5($k) . '.jpg';
							if (file_exists($cachePosterUrl))
								shell_exec('rm -rf "'.$cachePosterUrl.'" &');
							if (file_exists($arrCachePath))
								shell_exec('rm -rf "'.$arrCachePath.'" &');
						}
					}
					if (isset($this->casheHistArr)){
						$cUrl = MediaURL::decode($this->history_mUrl);
						$arrCachePath = $cachePath . md5($cUrl->movie_id . $cUrl->class_name) . '.srz';
						if (!file_exists($arrCachePath)){
							$cachePosterUrl = $cachePath . md5($cUrl->movie_id . $cUrl->class_name) . '.jpg';
							if (!file_exists($arrCachePath))
								HD::imgCache($cUrl->poster_url, md5($cUrl->movie_id . $cUrl->class_name) . '.jpg');
							HD::pluginCache(serialize($this->casheHistArr), md5($cUrl->movie_id . $cUrl->class_name) . '.srz');
						}
					}
				}
				$itemsHistory = array_slice($itemsHistory, 0, 310);
				HD::saveItemsArr('itemsHistory', $itemsHistory);
			}
			$state = HD::parseExtCommandState();
			$this->playback_info[$series_id]['d'] = $state['playback_duration'];
			$this->playback_info[$series_id]['p'] = $state['playback_position'];
			HD::saveItemsTmpArr('viewedItemsTmpArr', $this->playback_info);
			$prsnt = 0;
			if ($this->playback_info[$series_id]['d'] != 0)
				$prsnt =  round(($this->playback_info[$series_id]['p'] / $this->playback_info[$series_id]['d']) * 100);
			if ($this->sleepAlert == false){
				if (($this->sleepAfter == true)&&($prsnt > 98)){
					$params['sleepNotification'] = true;
					$post_action = UserInputHandlerRegistry::create_action($this, 'key_c_yellow', $params);
				}else{
					$ut = HD::getUnixTimestamp ();
					$cron_arr = HD::actionCrontabs();
					foreach ($cron_arr as $k => $v){
						if (preg_match('|\#sleep\s(\d*)|ms', $v, $matches)){
							if (($matches[1] - $ut < 0 )){
								unset ($cron_arr[$k]);
								$cronStr = implode("\n", $cron_arr);
								HD::actionCrontabs ($cronStr);
							}else if (($matches[1] - $ut < 120 )){
								$this->sleepAlert = true;
								$params['sleepNotification'] = true;
								$post_action = UserInputHandlerRegistry::create_action($this, 'key_c_yellow', $params);
							}
						}
					}
				}
			}
			
			if ($this->videoInfoShow == true){
				$file_info = $this->series_list[$user_input->plugin_vod_series_ndx]->file_info;
				if (isset($file_info['torrentId']))
					$tId = $file_info['torrentId'];
				else
					$tId = $user_input->plugin_vod_id;
				$class_name = $file_info['class_name'];
				if (isset($file_info['torrentClassName']))
					$class_name = $file_info['torrentClassName'];
				return $class_name::getInfoRend($tId, $actions, $post_action);
			}
			if (self::$timeInfoShow == true){
				if (self::$timeInfoShow != 'off'){
					self::$timeInfoShow['prsnt'] = $prsnt;
					self::$timeInfoShow['remainingTime'] = HD::sec_format_duration($this->playback_info[$series_id]['d'] - $this->playback_info[$series_id]['p'], false, true);
					return HD::timeInfoShowRend(self::$timeInfoShow, $actions, $post_action);
				}else if (self::$timeInfoShow == 'off'){
					self::$timeInfoShow = false;
					$comps = array();
					return ActionFactory::update_osd($comps, ActionFactory::change_behaviour($actions, 500, $post_action));
				}
			}
			return  ActionFactory::change_behaviour($actions, 5000, $post_action);
		}

		if ($user_input->control_id == 'playback_stop'){
			if (preg_match("/\.(?:mp3|wav|og(?:g|a)|flac|midi?|rm|m4a|aac|wma|mka|ape|jpg|png)$/i",$this->series_list[$user_input->plugin_vod_series_ndx]->playback_url))
				return null;
			$parent_media_url = urldecode($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['parent_media_url']);
			if (isset($this->playback_info)){
				$media_url = MediaURL::decode($parent_media_url);
				$viewedItemsArr = HD::getItemsArr('viewedItemsArr');
				if (isset($this->currentFav)){
					if (isset($this->mShowsA))
						if (isset($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['myShowsShowID']))
							VideoInfo::manageCheckEpisode_myshows($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['myShowsShowID'], $this->mShowsA);
					$upViewed = (HD::getItemStr('upViewed') != '') ? HD::getItemStr('upViewed') : 1;
					$currentFav = MediaURL::decode($this->currentFav);
					if ($upViewed == 1){
						$itemsFavorites = HD::getItemsArr($currentFav->class_name);
						if (isset($itemsFavorites[$this->currentFavItem])){
							if ($itemsFavorites[$this->currentFavItem] != 0){
								unset ($itemsFavorites[$this->currentFavItem]);
								$flipFavorites = array_flip ($itemsFavorites);
								array_unshift ($flipFavorites, $this->currentFavItem);
								$flipFavorites = array_values($flipFavorites);
								$itemsFavorites = array_flip ($flipFavorites);
								if (HD::saveItemsArr($currentFav->class_name, $itemsFavorites) != true)
									hd_print($currentFav->class_name . 'save err!');
								else
									$invalArr[] = $this->currentFav;
							}
						}
					}
					$saveFoldersArr = HD::getItemsArr($currentFav->class_name . '_saveFoldersArr');
				}
				foreach ($this->playback_info as $k => $v){
					$viewedItemsArr[$k] = $v;
					if (isset($saveFoldersArr)){
						if (isset($saveFoldersArr[$media_url->movie_id]['series'][md5($parent_media_url)])){
							if (!isset($saveFoldersArr[$media_url->movie_id]['series'][md5($parent_media_url)][$k])){
								$saveFoldersArr[$media_url->movie_id]['series'][md5($parent_media_url)][$k] = 1;
								$sfa = 1;
							}
						}
						if (isset($this->currentFav)){
							if (isset(SmartConfig::$newFavItems[$currentFav->class_name][$media_url->movie_id][$k])){
								unset(SmartConfig::$newFavItems[$currentFav->class_name][$media_url->movie_id][$k]);
								$sfa2 = 1;
							}
						}
					}
				}
				if (isset($sfa))
					HD::saveItemsArr($currentFav->class_name . '_saveFoldersArr', $saveFoldersArr);
				if (isset($sfa2))
					$invalArr[] = $this->currentFav;
				unset ($this->playback_info);
				HD::saveItemsArr('viewedItemsArr', $viewedItemsArr);
			}
			
			if (($this->sleepAfter == true)&& ($this->delayTimer == false)){
				exec ('echo A15EBF00 > /proc/ir/button');
				return null;
			}else if ($this->sleepAfter == true){
				$this->delayTimer = false;
				$this->sleepAlert = false;
			}
			$actions = $this->get_movie_actions();
			if ($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['class_name'] == 'PeerflixServer'){
				$allPeerflixDload = (HD::getItemStr('allPeerflixDload',true) !='') ? HD::getItemStr('allPeerflixDload', true) : 0;
				if ($allPeerflixDload == 1)
					PeerflixServer::peerflixServerActionDloadAll($user_input->plugin_vod_id, 'start');
			}
			if (isset($this->currentFav))
				if (isset($this->mShowsA))
					myShows_Class::getWatchedMyShows('clear');
			$action = null;
			if (self::$timeInfoShow == true){
				// self::$timeInfoShow = false;
				$comps = array();
				$action = ActionFactory::invalidate_folders(array($parent_media_url), $action);
				$action = ActionFactory::update_osd($comps, ActionFactory::change_behaviour($actions, 500, $action));
			}
			if ((defined('GUI_EVENT_MENU_PLAYBACK_FINISH') === true)&&(VodSeriesListScreen::$downViewed == true))
				VodSeriesListScreen::$series_ndx = $user_input->plugin_vod_series_ndx;
			$invalArr[] = $parent_media_url;
			return ActionFactory::invalidate_folders($invalArr, $action);
		}
		
		if ($user_input->control_id == 'saveTime'){
			if (isset($user_input->offon)){
				$saveTime = (HD::getItemStr('saveTime') !='') ? HD::getItemStr('saveTime') : 0;
				if ($saveTime == 0){
					$capt ='Время старта: Включено';
					$msg = 'Что бы указать с какого времени запускать все серии сезона нажмите красную А.';
					HD::saveItemStr('saveTime', 1);
				}else{
					$capt ='Время старта: Выключено';
					$msg = 'Все метки времени очищены';
					HD::saveItemStr('saveTime', 0);
				}
				$actions = $this->get_movie_actions();
				$do = UserInputHandlerRegistry::create_action($this, 'key_d_blue');
				return ActionFactory::show_title_dialog($capt,$do,$msg,800,1);
			}
			if (isset($user_input->clearAll)){
				$saveSeasonTimes = array();
				HD::saveItemsArr('saveSeasonTimes', $saveSeasonTimes);
				return ActionFactory::show_title_dialog('Все время старта очищены!');
			}
			if (isset($user_input->clearId)){
				$saveSeasonTimes = HD::getItemsArr('saveSeasonTimes');
				$tmp = $saveSeasonTimes[$user_input->clearId];
				unset($saveSeasonTimes[$user_input->clearId]);
				HD::saveItemsArr('saveSeasonTimes', $saveSeasonTimes);
				return ActionFactory::show_title_dialog('Удалено', null, 'Cтарт видео с:' . HD::sec_format_duration($tmp));
			}
		}
		if ($user_input->control_id == 'key_d_blue'){
			$defs = array();
			if (isset($this->class_info['dbIDs']['kpID'])){
				$add_action = UserInputHandlerRegistry::create_action($this, 'classAction',  array('action' => 'reviews', 'indiClass' => 'Kinopoisk_Class', 'kpID'=> $this->class_info['dbIDs']['kpID']));
				ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Отзывы Кинопоиск', 600, $add_action);
			}
			if ((isset($this->mShowsA))&&(isset($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['myShowsShowID']))){
				$add_action = UserInputHandlerRegistry::create_action($this, 'classAction',  array('action' => 'reviews', 'indiClass' => 'myShows_Class', 'mShowsA'=> $this->mShowsA, 'myShowsID' => $this->class_info["dbIDs"]["myShowsID"], 'myShowsShowID' => $this->series_list[$user_input->plugin_vod_series_ndx]->file_info['myShowsShowID'], 'series_name' => $this->series_list[$user_input->plugin_vod_series_ndx]->name, 'favMyShows' => $this->currentFavItem));
				ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Отзывы myShows к эпизоду', 600, $add_action);
			}
			$add_action = UserInputHandlerRegistry::create_action($this, 'key_c_yellow');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Sleep таймер', 600, $add_action);

			$saveTime = (HD::getItemStr('saveTime') !='') ? HD::getItemStr('saveTime') : 0;
			if ($saveTime == 1){
				$saveSeasonTimes = HD::getItemsArr('saveSeasonTimes');
				if (isset($saveSeasonTimes[$user_input->plugin_vod_id])){
					$params['clearId'] = $user_input->plugin_vod_id;
					$add_action = UserInputHandlerRegistry::create_action($this, 'saveTime', $params);
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Удалить старт с: ' . HD::sec_format_duration($saveSeasonTimes[$user_input->plugin_vod_id]), 600, $add_action);
				}
				if (count($saveSeasonTimes) > 0){
					$params['clearAll'] = true;
					$add_action = UserInputHandlerRegistry::create_action($this, 'saveTime', $params);
					ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Очистить все время старта', 600, $add_action);
				}
				$msg = 'Выключить время старта';
			}else
				$msg = 'Включить время старта';
			$params['offon'] = 1;
			$add_action = UserInputHandlerRegistry::create_action($this, 'saveTime', $params);
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', $msg, 600, $add_action);
			
			$add_action = UserInputHandlerRegistry::create_action($this, 'setupTimeInfoShowRend');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'_other_works', 'Часы', 600, $add_action);
			
			return ActionFactory::show_dialog("Меню", $defs, true, 0);
		}
		if ($user_input->control_id == 'setupTimeInfoShowRend'){
			$timeInfoShowRendSettings = HD::getItemsArr('timeInfoShowRendSettings', true);
			if(isset($user_input->timeInfoColor)){
				$timeInfoShowRendSettings['color'] = $user_input->timeInfoColor;
				$timeInfoShowRendSettings['halo'] = $user_input->halo;
				$timeInfoShowRendSettings['pluginFont'] = $user_input->pluginFont;
				$timeInfoShowRendSettings['position'] = $user_input->position;
				$timeInfoShowRendSettings['background'] = $user_input->background;
				$timeInfoShowRendSettings['clock'] = $user_input->clock;
				$timeInfoShowRendSettings['showPrsntTime'] = $user_input->showPrsntTime;
				HD::saveItemsArr('timeInfoShowRendSettings', $timeInfoShowRendSettings, true);
				if ($timeInfoShowRendSettings['position'] == 4)
					$timeInfoShowRendSettings['position'] = rand (0,3);
				if ($user_input->clock == 0){
					self::$timeInfoShow = 'off';
				}else if ($user_input->clock > 1)
					self::$timeInfoShow = $timeInfoShowRendSettings;
				else if (self::$timeInfoShow == true)
					self::$timeInfoShow = $timeInfoShowRendSettings;
				return null;
			}
			$clock = isset($timeInfoShowRendSettings['clock']) ? $timeInfoShowRendSettings['clock'] : 0;
			$halo = isset($timeInfoShowRendSettings['halo']) ? $timeInfoShowRendSettings['halo'] : 0;
			$color = isset($timeInfoShowRendSettings['color']) ? $timeInfoShowRendSettings['color'] : 14;
			$background = isset($timeInfoShowRendSettings['background']) ? $timeInfoShowRendSettings['background'] : 0;
			$pluginFont = isset($timeInfoShowRendSettings['pluginFont']) ? $timeInfoShowRendSettings['pluginFont'] : 'normal';
			$position = isset($timeInfoShowRendSettings['position']) ? $timeInfoShowRendSettings['position'] : 0;
			$showPrsntTime = isset($timeInfoShowRendSettings['showPrsntTime']) ? $timeInfoShowRendSettings['showPrsntTime'] : 0;
			// $add_params = null;
			$add_params['control_id']= $user_input->control_id;
			$w1 = 650;
			$t2 = '';
			for ($i = 0; $i<24; $i++){
				$color_ops[$i] = $i;
				$t2 .= '<text color="'.$i.'" size="small">'.$i.'|</text>';
			}
			ControlFactory::add_smart_label($defs, "",$t2);
			ControlFactory::add_combobox($defs, $this, $add_params,
				'timeInfoColor', '',
				$color, $color_ops, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Цвет</text>');
			$halo_ops[1] = 'Да';
			$halo_ops[0] = 'Нет';
			ControlFactory::add_vgap($defs, -75);
			ControlFactory::add_combobox($defs, $this, $add_params,
				'halo', '',
				$halo, $halo_ops, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Halo</text>');
			
			ControlFactory::add_vgap($defs, -75);
			$showPrsntOps[0] = 'Нет';
			$showPrsntOps[1] = '% просмотра';
			$showPrsntOps[2] = 'Оставшееся время';
			ControlFactory::add_combobox($defs, $this, $add_params,
				'showPrsntTime', '',
				$showPrsntTime, $showPrsntOps, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Отображать</text>');
			
			ControlFactory::add_vgap($defs, -75);
			ControlFactory::add_combobox($defs, $this, $add_params,
				'background', '',
				$background, $halo_ops, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Бекграунд</text>');
			ControlFactory::add_vgap($defs, -75);
			$pluginFont_ops['normal'] = 'NORMAL';
			$pluginFont_ops['small'] = 'SMALL';
			ControlFactory::add_combobox($defs, $this, $add_params,
				'pluginFont', '',
				$pluginFont, $pluginFont_ops, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Размер шрифта</text>');
			
			ControlFactory::add_vgap($defs, -75);
			$_ops[0] = 'Левый Верхний Угол';
			$_ops[1] = 'Левый Нижний Угол';
			$_ops[2] = 'Правый Верхний Угол';
			$_ops[3] = 'Правый Нижний Угол';
			$_ops[4] = 'Случайно';
			ControlFactory::add_combobox($defs, $this, $add_params,
				'position', '',
				$position, $_ops, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Расположение</text>');
			ControlFactory::add_vgap($defs, -75);
			
			$clock_ops[0] = 'Нет';
			$clock_ops[1] = 'Кнопка CLEAR';
			$clock_ops[2] = 'Всегда включены + CLEAR';
			$clock_ops[3] = 'Всегда включены';
			ControlFactory::add_combobox($defs, $this, $add_params,
				'clock', '',
				$clock, $clock_ops, $w1, false, false
			);
			ControlFactory::add_smart_label($defs,"", '<gap width="' . ($w1 + 20) . '"/><text dy="-69" color="'.$color.'">Отображение</text>');
			ControlFactory::add_vgap($defs, -75);
			$action = UserInputHandlerRegistry::create_action($this, 'setupTimeInfoShowRend');
			ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription', 'Применить', $w1, $action);
			return ActionFactory::show_dialog('Часы', $defs, true, $w1 + 450);
		}
		if ($user_input->control_id == 'key_clear'){
			$actions = $this->get_movie_actions();
			if ((self::$timeInfoShow == false)||(self::$timeInfoShow == 'off')){
				$series_id = $this->series_list[$user_input->plugin_vod_series_ndx]->id;
				$timeInfoShowRendSettings = HD::getItemsArr('timeInfoShowRendSettings', true);
				if ($timeInfoShowRendSettings['position'] == 4)
					$timeInfoShowRendSettings['position'] = rand (0,3);
				$prsnt = 0;
				if ((intval($this->playback_info[$series_id]['p'])>0)&&(intval($this->playback_info[$series_id]['d'])>0))
					$prsnt =  round(($this->playback_info[$series_id]['p'] / $this->playback_info[$series_id]['d']) * 100);
				$timeInfoShowRendSettings['prsnt'] = $prsnt;
				$timeInfoShowRendSettings['remainingTime'] = HD::sec_format_duration($this->playback_info[$series_id]['d'] - $this->playback_info[$series_id]['p'], false, true);
				self::$timeInfoShow = $timeInfoShowRendSettings;
				return HD::timeInfoShowRend(self::$timeInfoShow, $actions);
			}else{
				self::$timeInfoShow = false;
				$comps = array();
				return ActionFactory::update_osd($comps, ActionFactory::change_behaviour($actions, 500));
			}
		}
		if ($user_input->control_id == 'key_c_yellow'){
			$sleepNotification = false;
			if (isset($user_input->sleepNotification))
				$sleepNotification = true;
			return $this->sleep_defs($user_input, $sleepNotification);
		}
		if ($user_input->control_id == 'playback_going_to_switch'){
			if ($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['class_name'] == 'TransmissionUPnP')
				if($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['percent'] != 100)
					TransmissionUPnP::setTransmissionActionPriority($this->series_list[$user_input->plugin_vod_series_ndx]->file_info['torrent_id'], $this->series_list[$user_input->plugin_vod_series_ndx]->file_info['file_id']);
		}			
		if ($user_input->control_id == 'playback_switched'){
			if ($this->ims == 0){
				$saveSeasonTimes = HD::getItemsArr('saveSeasonTimes');
				if (isset($saveSeasonTimes[$user_input->plugin_vod_id])){
					$playback_state = $i = 0;
                    $port = HD::http_local_port();
					do {
						usleep(50000);
						$inf = file_get_contents("http://127.0.0.1:$port/cgi-bin/do?cmd=status");
						if (preg_match('|playback_state" value="(.+?)"|', $inf, $m))
							$playback_state = $m[1];	
						$i++;
					}while (($i < 130)&&($playback_state != 'playing'));
					if (HD::is_android() == true)
						usleep(800000);
					file_get_contents("http://127.0.0.1:$port/cgi-bin/do?cmd=set_playback_state&position=" . $saveSeasonTimes[$user_input->plugin_vod_id] . '&speed=256');						
				}
			} 
		}
		if ($user_input->control_id == 'key_a_red'){
			$state = HD::parseExtCommandState();
			$playback_position = $state['playback_position'];
			$saveSeasonTimes = HD::getItemsArr('saveSeasonTimes');
			$saveSeasonTimes[$user_input->plugin_vod_id] = $playback_position;
			HD::saveItemsArr('saveSeasonTimes', $saveSeasonTimes);
			return ActionFactory::show_title_dialog('Сохранено', null, 'Cтарт видео с: ' . HD::sec_format_duration($playback_position));
		}
		if ($user_input->control_id == 'key_b_green'){
			$file_info = $this->series_list[$user_input->plugin_vod_series_ndx]->file_info;
			if (($file_info['class_name'] == 'TorrServer')||
			($file_info['class_name'] == 'PeerflixServer')||
			(isset($file_info['torrentClassName']))){
				$actions = $this->get_movie_actions();
				if ($this->videoInfoShow == true){
					$comps = array();
					$this->videoInfoShow = false;
					return ActionFactory::update_osd($comps, ActionFactory::change_behaviour($actions, 500));
				}else{
					$this->videoInfoShow = true;
					if (isset($file_info['torrentId']))
						$tId = $file_info['torrentId'];
					else
						$tId = $user_input->plugin_vod_id;
					$class_name = $file_info['class_name'];
					if (isset($file_info['torrentClassName']))
						$class_name = $file_info['torrentClassName'];
					return $class_name::getInfoRend($tId, $actions, $post_action=null);
				}
			}
		}
	}
	
	protected function get_movie_actions(){
		if (defined('GUI_EVENT_PLAYBACK_GOING_TO_SWITCH') === true)
			$actions[GUI_EVENT_PLAYBACK_GOING_TO_SWITCH] = UserInputHandlerRegistry::create_action($this, 'playback_going_to_switch');
		if (((HD::getItemStr('saveTime') !='') ? HD::getItemStr('saveTime') : 0) == 1){
			$actions[GUI_EVENT_PLAYBACK_SWITCHED] = UserInputHandlerRegistry::create_action($this, 'playback_switched');
			$actions[GUI_EVENT_KEY_A_RED]	= UserInputHandlerRegistry::create_action($this, 'key_a_red');
		}
		$actions[GUI_EVENT_KEY_B_GREEN] = UserInputHandlerRegistry::create_action($this, 'key_b_green');
		
		$actions[GUI_EVENT_KEY_C_YELLOW] = UserInputHandlerRegistry::create_action($this, 'key_c_yellow');
		$actions[GUI_EVENT_KEY_D_BLUE] = UserInputHandlerRegistry::create_action($this, 'key_d_blue');
		$actions[GUI_EVENT_TIMER] = UserInputHandlerRegistry::create_action($this, 'event_timer');
		$actions[GUI_EVENT_PLAYBACK_STOP] = UserInputHandlerRegistry::create_action($this, 'playback_stop');
		
		$timeInfoShowRendSettings = HD::getItemsArr('timeInfoShowRendSettings', true);
		$clock = isset($timeInfoShowRendSettings['clock']) ? $timeInfoShowRendSettings['clock'] : 0;
		if (($clock == 1)||($clock == 2))
			$actions[GUI_EVENT_KEY_CLEAR] = UserInputHandlerRegistry::create_action($this, 'key_clear');
		return $actions;
	}
	
	public function sleep_defs($user_input, $sleepNotification){
		$attrs = $defs = array();
		$ut = HD::getUnixTimestamp ();
		$cron_arr = HD::actionCrontabs();
		if (isset($user_input->saveSleepTime)){
			if (isset($user_input->delayTimer)){
				$this->sleepAlert = true;
				$this->delayTimer = true;
				return null;
			}
			$this->sleepAfter = false;
			foreach ($cron_arr as $k => $v){
				if (preg_match('|\#sleep|ms', $v)){
					unset ($cron_arr[$k]);
					$nsct = true;
				}
			}
			if ($user_input->sleep_time == 'after'){
				$this->sleepAfter = true;
				$this->sleepAlert = false;
			}else if ($user_input->sleep_time != 0){
				$this->sleepAlert = false;
				$sleep_time = $ut + $user_input->sleep_time;
				$cron_arr[] =  date("i H d m", $sleep_time) . " * echo A15EBF00 > /proc/ir/button #sleep $sleep_time";
				$nsct = true;
			}
			if (isset($nsct)){
				$cronStr = implode("\n", $cron_arr);
				HD::actionCrontabs ($cronStr);
			}
			return null;
		}
		if ($sleepNotification==true){
			$attrs['timer'] = ActionFactory::timer(30000);
			$attrs['actions'] = array(GUI_EVENT_TIMER => ActionFactory::close_dialog());
			if (self::$timeInfoShow == true)
				self::$timeInfoShow = 'off';
		}
		$sleep_str = false;
		foreach ($cron_arr as $k => $v){
			if (preg_match('|\#sleep\s(\d*)|ms', $v, $matches)){
				if ($ut > $matches[1]){
					unset ($cron_arr[$k]);
					$cronStr = implode("\n", $cron_arr);
					HD::actionCrontabs ($cronStr);
				}else
					$sleep_str = true;
			}
		}
		$sleep_ops [0] = 'Установите выключение';
		$sleep_ops ['after'] = 'После текущей серии';
		$sleep_ops [900] = '15 минут';
		$sleep_ops [1200] = '20 минут';
		$sleep_ops [1800] = '30 минут';
		$sleep_ops [2700] = '45 минут';
		$sleep_ops [3600] = '1 час';
		$sleep_ops [4500] = '1 час 15 минут';
		$sleep_ops [5400] = '1 час 30 минут';
		$sleep_ops [6300] = '1 час 45 минут';
		$sleep_ops [7200] = '2 часа';
		$sleep_ops [9000] = '2 часа 30 минут';
		$sleep_ops [10800] = '3 часа';
		if ($this->sleepAfter == true)
			ControlFactory::add_label($defs, '', "Выключение: После текущей серии");
		else if ($sleep_str == true)
			ControlFactory::add_label($defs, '', "Выключение: через " . HD::sec_format_duration($matches[1] - $ut));
		
		ControlFactory::add_combobox($defs, $this, null,
			'sleep_time', '',
			0, $sleep_ops, 650, false, false
		);
		ControlFactory::add_vgap($defs, 20);
		$add_params['saveSleepTime'] = true;
		ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, $user_input->control_id, 'Применить', 450);
		
		if ($this->sleepAfter == true){
			$add_params['delayTimer'] = 1;
			ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, $user_input->control_id, 'Отложить таймер', 450);
			$attrs['initial_sel_ndx'] = 2;
			$sleep_str = true;
		}
		if ($sleep_str == true){
			unset($add_params['delayTimer']);
			ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, $user_input->control_id, 'Очистить таймеры', 450);
		}
		$currentTime = HD::getCurrentTime('G:i');
		return ActionFactory::show_dialog($currentTime . " Sleep таймер", $defs, true, 750, $attrs);
	}
}

?>
