<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/screen.php';
require_once 'lib/vod/vod.php';
require_once 'lib/vod/vod_series_list_screen.php';

class VodMovieScreen implements Screen, UserInputHandler
{
    const ID = 'vod_movie';

    public static function get_media_url_str($class_name, $arr=null){
		$arr['screen_id'] = self::ID;			
		$arr['class_name'] = $class_name;
        return MediaURL::encode($arr);
    }

    ///////////////////////////////////////////////////////////////////////

    private $vod;
	private $mUrl;
	private $left_button_caption;
	private $movie;
    public function __construct(Vod $vod){
        $this->vod = $vod;

        UserInputHandlerRegistry::get_instance()->
            register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_id(){ return self::ID; }

    public function get_handler_id(){ return self::ID; }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view(MediaURL $media_url, &$plugin_cookies){
        $this->vod->folder_entered($media_url, $plugin_cookies);
		if ((isset($this->vod->seasons_media_url['movie_screen']))&&
		($this->vod->seasons_media_url['movie_screen'] != $media_url->get_raw_string()))
			$this->vod->seasons_media_url = null;
		if (isset($this->vod->seasons_media_url['seasons']))
			unset($this->vod->seasons_media_url['seasons']);
		$this->vod->seasons_media_url['movie_screen'] = $media_url->get_raw_string();
        $this->movie = $this->vod->get_loaded_movie($media_url, $plugin_cookies);
        if ($this->movie === null)
            return null;
		$paintBackground = (HD::getItemsArr('paintBackground') != array()) ? HD::getItemsArr('paintBackground') : array('paint_widget_background' => true, 'paint_content_box_background' => true, 'paint_path_box_background' => true, 'paint_details_box_background' => true);
		$class_name = $media_url->class_name;
		$mArr['movie_id'] = $this->movie->id;
		if (isset($media_url->links))
			$mArr['links'] = $media_url->links;
		if (isset($this->movie->class_info['goToScreen'])){
			$goToScreen = $this->movie->class_info['goToScreen'];
			$this->mUrl = $goToScreen::get_media_url_str($class_name, $mArr);
			if ($goToScreen == 'SmartVodTorrentsScreen')
				$this->left_button_caption = 'Торренты';
		}else{
			$this->mUrl = VodSourceListScreen::get_media_url_str($class_name, $mArr);
			$this->left_button_caption = 'Источники';
		}
		if (isset($this->vod->currentFav)){
			$currentFav = MediaURL::decode($this->vod->currentFav);
			$saveFoldersArr = HD::getItemsArr($currentFav->class_name . '_saveFoldersArr');
			if (isset($saveFoldersArr[$media_url->movie_id]['mUrl'])){
				$this->mUrl = $saveFoldersArr[$media_url->movie_id]['mUrl'];
				$this->left_button_caption = 'Просмотр';
			}
		}
		$backgroundShow = (HD::getItemStr('backgroundShow') != '') ? HD::getItemStr('backgroundShow') : 1;
		if (($backgroundShow == 1)&&(isset($this->movie->background_urls)))
			$background_url =  $this->movie->background_urls[array_rand($this->movie->background_urls)];
		else
			$background_url = SmartConfig::getPluginIco(self::ID . '_bg');
		
		$params[PluginFolderViewParams::paint_path_box] = false;
		$params[PluginFolderViewParams::paint_content_box_background] = $paintBackground['paint_content_box_background'];
		$params[PluginFolderViewParams::background_url] = $background_url;

		$movie_folder_view[PluginMovieFolderView::movie] = $this->movie->get_movie_array();
		$movie_folder_view[PluginMovieFolderView::has_right_button] = true;
		$movie_folder_view[PluginMovieFolderView::left_button_caption] = $this->left_button_caption;
		$movie_folder_view[PluginMovieFolderView::left_button_action] = UserInputHandlerRegistry::create_action($this, 'add_actions');
		$movie_folder_view[PluginMovieFolderView::right_button_caption] = "Действия";
		$movie_folder_view[PluginMovieFolderView::right_button_action] = UserInputHandlerRegistry::create_action($this, 'rightButtonAction');
		$movie_folder_view[PluginMovieFolderView::has_multiple_series] = true;
		// $movie_folder_view[PluginMovieFolderView::series_media_url] = $mUrl;
		$movie_folder_view[PluginMovieFolderView::params] = $params;

        return array(
            PluginFolderView::multiple_views_supported  => false,
            PluginFolderView::archive                   => null,
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_MOVIE,
            PluginFolderView::data                      => $movie_folder_view,
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public function handle_user_input(&$user_input, &$plugin_cookies){
		if ($user_input->control_id == 'ifx'){
			$this->vod->clear_cached_movie();
			return ActionFactory::invalidate_folders(array($user_input->parent_media_url));
		}
		if ($user_input->control_id == 'rightButtonAction'){
			$defs = array();
			hd_silence_warnings();
			if (isset($this->vod->mShowsA))
				$user_input->mShowsA = $this->vod->mShowsA;
			return SmartConfig::rightButtonAction ($this->movie, $user_input, $this->left_button_caption);
		}
		if ($user_input->control_id == 'classAction'){
			$defs = array();
			hd_silence_warnings();
			if ((isset($user_input->delMyShowId))||(isset($user_input->delSearchKpId))){
				$this->vod->clear_cached_movie();
			}
			if (isset($user_input->indiClass)){
				$class_name = $user_input->indiClass;
				return $class_name::classAction ($defs, $user_input);
			}
			return SmartConfig::classAction ($defs, $user_input);
		}
		if ($user_input->control_id == 'add_actions'){
			$caption = $this->left_button_caption;
			if ($caption == 'Просмотр'){
				$media_url = MediaURL::decode($this->mUrl);
				$caption = $media_url->caption;
			}
			$this->vod->seasons_media_url['screen_counter'] = 0;
			VodSourceListScreen::$filter = false;
			return ActionFactory::open_folder($this->mUrl, $caption);
		}
		return null;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
