<?php
require_once 'lib/vod/vod.php';
require_once 'lib/abstract_preloaded_regular_screen.php';
class VodRandomScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'vod_random';

    public static function get_media_url_str($class_name, $arr=null){
		$arr['screen_id'] = self::ID;			
		$arr['class_name'] = $class_name;
        return MediaURL::encode($arr);
    }

    ///////////////////////////////////////////////////////////////////////

    private $vod;
    public function __construct(Vod $vod){
        $this->vod = $vod;
        parent::__construct(self::ID,
           $this->get_folder_views());

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies){
		$class_name = $media_url->class_name;
		$params['event_key'] = 'enter';
		$actions[GUI_EVENT_KEY_ENTER]		= UserInputHandlerRegistry::create_action($this, 'add_actions',$params);
		$params['event_key'] = 'play';
		$actions[GUI_EVENT_KEY_PLAY] 		= UserInputHandlerRegistry::create_action($this, 'add_actions',$params);

		$actions[GUI_EVENT_KEY_POPUP_MENU] = UserInputHandlerRegistry::create_action($this, 'popup_menu');

		$add_action = UserInputHandlerRegistry::create_action($this, 'invalidate');
        $add_action['caption'] = 'Обновить!';
        $actions[GUI_EVENT_KEY_B_GREEN] = $add_action;
		
		$actions[GUI_EVENT_KEY_SEARCH] = UserInputHandlerRegistry::create_action($this, 'classAction', array('action' => 'search'));
		
		$add_action = UserInputHandlerRegistry::create_action($this, 'classAction', array('action' => 'randomGenres', 'indiClass' => $class_name));
        $add_action['caption'] = 'Жанры';
        $actions[GUI_EVENT_KEY_C_YELLOW] = $add_action;

		$add_params['action'] = 'addToFav';
		$add_action = UserInputHandlerRegistry::create_action($this, 'classAction', $add_params);
		$add_action['caption'] = 'Избранное';
        $actions[GUI_EVENT_KEY_D_BLUE] = $add_action;

		$actions[GUI_EVENT_KEY_SETUP] = ActionFactory::open_folder('setup','Настройки');

		
		$actionsArr = $class_name::getScreenActionMapArr($media_url);
		foreach ($actionsArr as $k => $v){
			$params = false;
			if (isset($v['params']))
				$params = $v['params'];
			$add_action = UserInputHandlerRegistry::create_action($this, $v['create_action'], $params);
			if (isset($v['caption']))
				$add_action['caption'] = $v['caption'];
			$actions[$v['key']] = $add_action;
		}

		$useNumKey = HD::getItemsArr('useNumKey');
		foreach ($useNumKey as $k => $v){
			$params['openCat'] = $v;
			$actions[SmartConfig::$keyArr[$k]] = UserInputHandlerRegistry::create_action($this, 'add_actions', $params);
		}
		return $actions;
    }

    public function get_handler_id(){ return self::ID; }

    public function handle_user_input(&$user_input, &$plugin_cookies){
		if ($user_input->control_id == 'invalidate'){
			return ActionFactory::invalidate_folders(array($user_input->parent_media_url));		
		}
		if ($user_input->control_id == 'add_actions'){
			if (isset($user_input->openCat)){
				$user_input->selected_media_url = $user_input->openCat; 
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				return ActionFactory::open_folder($user_input->selected_media_url, $selected_media_url->caption);
			}
			return ActionFactory::open_folder();
		}
		if ($user_input->control_id == 'classAction'){
			$defs = array();
			hd_silence_warnings();
			if (isset($user_input->indiClass)){
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				$class_name = $selected_media_url->class_name;
				return $class_name::classAction ($defs, $user_input, $this->select_url);
			}
			return SmartConfig::classAction ($defs, $user_input, $this->select_url);
		}
		if ($user_input->control_id == 'popup_menu'){
			if (isset($user_input->selected_media_url)){
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				$class_name = $selected_media_url->class_name;
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'История Случайных',
					GuiMenuItemDef::action => ActionFactory::open_folder(
					VodHistoryScreen::get_media_url_str('randomMoviesHistory' . $class_name, array('caption' => 'История' .' - '. $media_url->caption)),
					'История'),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_restart')
				);
				$menu_items [] =  array(
					GuiMenuItemDef::is_separator => true,);
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'Поиск',
					GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', array('action' => 'search')),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_ssearch')
				);
				$menu_items [] =  array(
					GuiMenuItemDef::is_separator => true,);
				$selected_media_url = MediaURL::decode($user_input->selected_media_url);
				$class_name = $selected_media_url->class_name;
				if (preg_match("|_Class|",$class_name)){
					$mItems = $class_name::getPopUpMenuItems($selected_media_url, $user_input->parent_media_url);
					foreach ($mItems as $v){
						if ($v == 'is_separator')
							$menu_items [] =  array(
							GuiMenuItemDef::is_separator => true,);
						// else if ($v['params'] == 'classSetup')
							// $menu_items[] = array(
								// GuiMenuItemDef::caption => $v['caption'],
								// GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classSetup' . $class_name),
								// GuiMenuItemDef::icon_url => $v['icon_url'],
							// );
						else
							$menu_items[] = array(
								GuiMenuItemDef::caption => $v['caption'],
								GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $v['params']),
								GuiMenuItemDef::icon_url => $v['icon_url'],
							);
					}
				}
			}
			$menu_items [] =  array(
				GuiMenuItemDef::is_separator => true,);
			$add_params['action'] = 'background_actions';
			$menu_items[] = array(
				GuiMenuItemDef::caption => 'Задать бекграунд',
				GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
				GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_add_background')
			);
			if (file_exists(HD::get_data_path() . "/icons/" . self::ID . "_bg.jpg")){
				$add_params['action'] = 'background_actions';
				$add_params['del'] = true;
				$menu_items[] = array(
					GuiMenuItemDef::caption => 'Удалить бекграунд',
					GuiMenuItemDef::action => UserInputHandlerRegistry::create_action($this, 'classAction', $add_params),
					GuiMenuItemDef::icon_url => SmartConfig::getPluginIco('s_del_background')
				);
			}
			return ActionFactory::show_popup_menu($menu_items);
		}
        return null;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies){
        $this->vod->folder_entered($media_url, $plugin_cookies);
        $items = array();
		$view_idx = isset($plugin_cookies->{"screen.".self::ID.".view_idx"}) ? $plugin_cookies->{"screen.".self::ID.".view_idx"} : 0;
		$class_name = $media_url->class_name;
		$randomMovies = $class_name::getRandomMovies($media_url);
		$itemsHistory = HD::getItemsArr('randomMoviesHistory' . $class_name);
		foreach ($randomMovies as $movie){
			$mArr = array();
			$mArr['movie_id'] = $movie->id;
			$mArr['caption'] = $movie->name;
			$mArr['poster_url'] = $movie->poster_url;
			if (isset($movie->caption_original))
				$mArr['caption_original'] = $movie->caption_original;
			if (isset($movie->year))
				$mArr['year'] = $movie->year;
			// $mArr['info'] = $movie->info;
			if (isset($media_url->hCaptions))
				$mArr['hCaptions'] = $media_url->hCaptions;
			if (isset($media_url->caption))
				$mArr['hCaptions'][] = $media_url->caption;
			$mUrl = VodMovieScreen::get_media_url_str($class_name, $mArr);
			
			if ((isset($movie->left_top))||(isset($movie->right_top))||(isset($movie->left_bottom))||(isset($movie->right_bottom))||
			(isset($movie->left_top_img))||(isset($movie->right_top_img))||(isset($movie->left_bottom_img))||(isset($movie->right_bottom_img))){
				$geoArr = $imgArr = $txtArr = array();
				if ((defined('ViewItemParams::item_sticker') === true)&&($view_idx != 3)){
					if (isset($movie->left_top)){
						$txtArr[0] = $movie->left_top;
						$geoArr['txt_color'][0] = 23;
						$geoArr['txt_x'][0] = 15; 
						$geoArr['txt_y'][0] = 10;
						$geoArr['txt_halign'][0] = 'left';
					}
					if (isset($movie->right_top)){
						$txtArr[1] = $movie->right_top;
						$geoArr['txt_color'][1] = 23;
						$geoArr['txt_x'][1] = 15; 
						$geoArr['txt_y'][1] = 10;
						$geoArr['txt_halign'][1] = 'right';
					}
					if (isset($movie->left_bottom)){
						$txtArr[2] = $movie->left_bottom;
						$geoArr['txt_color'][2] = 23;
						$geoArr['txt_x'][2] = 15; 
						$geoArr['txt_y'][2] = 10;
						$geoArr['txt_halign'][2] = 'left';
						$geoArr['txt_valign'][2] = 'bottom';
					}
					if (isset($movie->right_bottom)){
						$txtArr[3] = $movie->right_bottom;
						$geoArr['txt_color'][3] = 23;
						$geoArr['txt_x'][3] = 15; 
						$geoArr['txt_y'][3] = 10;
						$geoArr['txt_halign'][3] = 'right';
						$geoArr['txt_valign'][3] = 'bottom';
					}
					if (isset($movie->left_top_img)){
						$imgArr[0] = $movie->left_top_img;
						$geoArr['img_x'][0] = 3; 
						$geoArr['img_y'][0] = 7;
						$geoArr['img_halign'][0] = 'left';
					}
					if (isset($movie->right_top_img)){
						$imgArr[1] = $movie->right_top_img;
						$geoArr['img_x'][1] = -2; 
						$geoArr['img_y'][1] = 2;
						$geoArr['img_halign'][1] = 'right';
					}
					if (isset($movie->left_bottom_img)){
						$imgArr[2] = $movie->left_bottom_img;
						$geoArr['img_x'][2] = 2; 
						$geoArr['img_y'][2] = 47;
						$geoArr['img_halign'][2] = 'left';
						$geoArr['img_valign'][2] = 'bottom';
					}
					if (isset($movie->right_bottom_img)){
						$imgArr[3] = $movie->right_bottom_img;
						$geoArr['img_x'][3] = 7; 
						$geoArr['img_y'][3] = 7;
						$geoArr['img_halign'][3] = 'right';
						$geoArr['img_valign'][3] = 'bottom';
					}
					$view_item_params[ViewItemParams::item_sticker] = json_encode(HD::get_sticker($imgArr, $txtArr, $geoArr));
				}
			}
			if((defined('ViewItemParams::item_extra_info'))&&($view_idx == 3)){
				if (isset($movie->extra_item)){
					$extra_item = array('type' => 'text', 'color' => '15', 'text' => $movie->extra_item);
					$item_extra_info = array(
						'items' => array(
						array('type' => 'gap', 'width' => 40),
						$extra_item,
					));
					$view_item_params[ViewItemParams::item_extra_info] = json_encode($item_extra_info);
					$view_item_params[ViewItemParams::item_caption_dy]=-10;
					$view_item_params[ViewItemParams::item_caption_sel_dy]=-10;
				}
			}
			$view_item_params[ViewItemParams::icon_path] = $movie->poster_url;
			$view_item_params[ViewItemParams::item_detailed_info] = $movie->info;
			if (isset($itemsHistory[$mUrl]))
				unset ($itemsHistory[$mUrl]);
			$tmpHist[$mUrl] = $movie->info;
            $items[] = array(
                PluginRegularFolderItem::media_url => $mUrl,
                PluginRegularFolderItem::caption => $movie->name,
                PluginRegularFolderItem::view_item_params => $view_item_params
            );
            $this->vod->set_cached_short_movie(new ShortMovie($movie->id, $movie->name, $movie->poster_url));
		}
		$itemsHistory = $tmpHist + $itemsHistory;
		$itemsHistory = array_slice($itemsHistory, 0, 310);
		HD::saveItemsArr('randomMoviesHistory' . $class_name, $itemsHistory);
        return $items;
    }
	public function get_folder_views(){
		$sandwich_cover = SmartConfig::getPluginIco('sandwich_cover2');
		$sandwich_base = SmartConfig::getPluginIco('poster_none');
		$sandwich_base2 = SmartConfig::getPluginIco('badge1');
		$background_path = SmartConfig::getPluginIco(self::ID . '_bg');
		$paintBackground = (HD::getItemsArr('paintBackground') != array()) ? HD::getItemsArr('paintBackground') : array('paint_widget_background' => true, 'paint_content_box_background' => true, 'paint_path_box_background' => true, 'paint_details_box_background' => true);
		$paintSandwich = (HD::getItemStr('paintSandwich') != '') ? (bool)HD::getItemStr('paintSandwich') : true;
		
		$arr[0][PluginRegularFolderView::async_icon_loading] = true;
		//$arr[0][PluginRegularFolderView::timer] = array(GuiTimerDef::delay_ms =>  1000);
		
		$arr[0][PluginRegularFolderView::view_params][ViewParams::num_cols] = 5;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::num_rows] = 2;
		// $arr[0][PluginRegularFolderView::view_params][ViewParams::orientation] = ORIENTATION_HORIZONTAL;
		// $arr[0][PluginRegularFolderView::view_params][ViewParams::paint_scrollbar] = false;
		// $arr[0][PluginRegularFolderView::view_params][ViewParams::cycle_mode_enabled] = true;
		// $arr[0][PluginRegularFolderView::view_params][ViewParams::cycle_mode_gap]=CYCLE_MODE_GAP_YES;
		// $arr[0][PluginRegularFolderView::view_params][ViewParams::cycle_mode_gap_width]=1100;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_details] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_item_info_in_details] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_auto_line_break] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_title_color] = 10;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_text_color] = 15;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_font_size] = (HD::getItemStr('detailedInfoFontSize') != '') ? HD::getItemStr('detailedInfoFontSize') : FONT_SIZE_SMALL;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_sandwich] = $paintSandwich;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_base] = $sandwich_base;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_mask] = 'cut_icon://{name=sandwich_mask}';
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_cover] = $sandwich_cover;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_width] = 206;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_height] = 300;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_icon_upscale_enabled] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::sandwich_icon_keep_aspect_ratio] = false;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_path] = $background_path;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_order] = 0;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_height] = 1080;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::background_width] = 1920;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::optimize_full_screen_background] = true;
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_widget_background] = $paintBackground['paint_widget_background'];
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_content_box_background] = $paintBackground['paint_content_box_background'];
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_path_box_background] = $paintBackground['paint_path_box_background'];
		$arr[0][PluginRegularFolderView::view_params][ViewParams::paint_details_box_background] = $paintBackground['paint_details_box_background'];
		
		$arr[0][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[0][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_width] = 206;
		$arr[0][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_height] = 300;
		$arr[0][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_path] = $sandwich_base2;
        
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_layout] = HALIGN_CENTER;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_valign] = VALIGN_CENTER;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_caption] = false;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_scale_factor] = 1.0;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_scale_factor] = 1.1;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_width] = 206;
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_height] = 300;
		if (defined('ViewItemParams::item_detailed_icon_width') === true){
			$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_width] = 200;
			$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_height] = 291;
		}
		$arr[0][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_margin_top] = 0;
		
		$arr[1][PluginRegularFolderView::async_icon_loading] = true;
		//$arr[1][PluginRegularFolderView::timer] = array(GuiTimerDef::delay_ms =>  1000);
		$arr[1][PluginRegularFolderView::view_params][ViewParams::num_cols] = 4;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::num_rows] = 2;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_details] = true;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_item_info_in_details] = true;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_auto_line_break] = true;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_title_color] = 10;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_text_color] = 15;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_font_size] = (HD::getItemStr('detailedInfoFontSize') != '') ? HD::getItemStr('detailedInfoFontSize') : FONT_SIZE_SMALL;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_sandwich] = $paintSandwich;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_base] = $sandwich_base;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_mask] = 'cut_icon://{name=sandwich_mask}';
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_cover] = $sandwich_cover;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_width] = 206;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_height] = 300;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_icon_upscale_enabled] = true;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::sandwich_icon_keep_aspect_ratio] = false;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::background_path] = $background_path;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::background_order] = 0;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::background_height] = 1080;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::background_width] = 1920;
		$arr[1][PluginRegularFolderView::view_params][ViewParams::optimize_full_screen_background] = true;
		if (defined('ViewParams::details_box_width') === true){
			$arr[1][PluginRegularFolderView::view_params][ViewParams::details_box_width] = 635;
			$arr[1][PluginRegularFolderView::view_params][ViewParams::content_box_x] = 100;
			$arr[1][PluginRegularFolderView::view_params][ViewParams::content_box_width] = 1070;
		}
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_widget_background] = $paintBackground['paint_widget_background'];
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_content_box_background] = $paintBackground['paint_content_box_background'];
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_path_box_background] = $paintBackground['paint_path_box_background'];
		$arr[1][PluginRegularFolderView::view_params][ViewParams::paint_details_box_background] = $paintBackground['paint_details_box_background'];
		
		$arr[1][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[1][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_width] = 206;
		$arr[1][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_height] = 300;
		$arr[1][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_path] = $sandwich_base2;
        
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_layout] = HALIGN_CENTER;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_valign] = VALIGN_CENTER;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_caption] = false;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_scale_factor] = 1.0;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_scale_factor] = 1.1;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_width] = 206;
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_height] = 300;
		if (defined('ViewItemParams::item_detailed_icon_width') === true){
			$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_width] = 200;
			$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_height] = 291;
		}
		$arr[1][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_margin_top] = 0;
		
		$arr[2][PluginRegularFolderView::async_icon_loading] = true;
		//$arr[2][PluginRegularFolderView::timer] = array(GuiTimerDef::delay_ms =>  1000);
		$arr[2][PluginRegularFolderView::view_params][ViewParams::num_cols] = 1;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::num_rows] = 4;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_details] = true;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_item_info_in_details] = true;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_auto_line_break] = true;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_title_color] = 10;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_text_color] = 15;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_font_size] = (HD::getItemStr('detailedInfoFontSize') != '') ? HD::getItemStr('detailedInfoFontSize') : FONT_SIZE_SMALL;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::background_path] = $background_path;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::background_order] = 0;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::background_height] = 1080;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::background_width] = 1920;
		$arr[2][PluginRegularFolderView::view_params][ViewParams::optimize_full_screen_background] = true;
		if (defined('ViewParams::paint_icon_badge_box') === true){
			$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_icon_badge_box] = true;
			$arr[2][PluginRegularFolderView::view_params][ViewParams::icon_badge_box_width] = 105;
			$arr[2][PluginRegularFolderView::view_params][ViewParams::icon_badge_box_height] = 155;
			$arr[2][PluginRegularFolderView::view_params][ViewParams::icon_badge_box_dx] = -580;
			$arr[2][PluginRegularFolderView::view_params][ViewParams::icon_badge_box_sel_width] = 110;
			$arr[2][PluginRegularFolderView::view_params][ViewParams::icon_badge_box_sel_height] = 160;
		}
		$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_widget_background] = $paintBackground['paint_widget_background'];
		$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_content_box_background] = $paintBackground['paint_content_box_background'];
		$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_path_box_background] = $paintBackground['paint_path_box_background'];
		$arr[2][PluginRegularFolderView::view_params][ViewParams::paint_details_box_background] = $paintBackground['paint_details_box_background'];
		
		$arr[2][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[2][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_width] = 110;
		$arr[2][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_height] = 160;
		$arr[2][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_path] = $sandwich_base2;
		
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_layout] = HALIGN_LEFT;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_valign] = VALIGN_CENTER;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_caption] = false;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_scale_factor] = 1.0;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_scale_factor] = 1.1;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_width] = 110;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_height] = 160;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_dx] = 4;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_dy] = -7;
		if (defined('ViewItemParams::item_detailed_icon_width') === true){
			$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_width] = 200;
			$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_height] = 291;
		}
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_margin_top] = 0;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_caption] = true;
		$arr[2][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_caption_width] = 1030;
		
		$arr[3][PluginRegularFolderView::async_icon_loading] = true;
		//$arr[3][PluginRegularFolderView::timer] = array(GuiTimerDef::delay_ms =>  1000);
		$arr[3][PluginRegularFolderView::view_params][ViewParams::num_cols] = 1;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::num_rows] = 10;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::paint_details] = true;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::paint_item_info_in_details] = true;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_auto_line_break] = true;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_title_color] = 10;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_text_color] = 15;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::item_detailed_info_font_size] = (HD::getItemStr('detailedInfoFontSize') != '') ? HD::getItemStr('detailedInfoFontSize') : FONT_SIZE_SMALL;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::background_path] = $background_path;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::background_order] = 0;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::background_height] = 1080;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::background_width] = 1920;
		$arr[3][PluginRegularFolderView::view_params][ViewParams::optimize_full_screen_background] = true;
		
		$arr[3][PluginRegularFolderView::view_params][ViewParams::paint_widget_background] = $paintBackground['paint_widget_background'];
		$arr[3][PluginRegularFolderView::view_params][ViewParams::paint_content_box_background] = $paintBackground['paint_content_box_background'];
		$arr[3][PluginRegularFolderView::view_params][ViewParams::paint_path_box_background] = $paintBackground['paint_path_box_background'];
		$arr[3][PluginRegularFolderView::view_params][ViewParams::paint_details_box_background] = $paintBackground['paint_details_box_background'];
		
		$arr[3][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[3][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_width] = 34;
		$arr[3][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_height] = 50;
		$arr[3][PluginRegularFolderView::not_loaded_view_item_params][ViewItemParams::icon_path] = $sandwich_base2;
		
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_icon] = true;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_layout] = HALIGN_LEFT;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_valign] = VALIGN_CENTER;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_caption] = false;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_scale_factor] = 1.0;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_scale_factor] = 1.1;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_width] = 34;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_height] = 50;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_dx] = 5;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_dy] = -5;
		if (defined('ViewItemParams::item_detailed_icon_width') === true){
			$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_width] = 200;
			$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_detailed_icon_height] = 291;
		}
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::icon_sel_margin_top] = 0;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_paint_caption] = true;
		$arr[3][PluginRegularFolderView::base_view_item_params][ViewItemParams::item_caption_width] = 1100;

		return $arr;
    }
    public function get_archive(MediaURL $media_url){
        return $this->vod->get_archive($media_url);
    }
}

?>
